@@ -1,6 +1,30 @@
+
+## Authors 
+## Martin Schlather, schlather@math.uni-mannheim.de
+##
+##
+## Copyright (C) 2012 -- 2015 Alexander Malinowski & Martin Schlather
+## Copyright (C) 2015 -- 2017 Martin Schlather
+##
+## This program is free software; you can redistribute it and/or
+## modify it under the terms of the GNU General Public License
+## as published by the Free Software Foundation; either version 3
+## of the License, or (at your option) any later version.
+##
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+##
+## You should have received a copy of the GNU General Public License
+## along with this program; if not, write to the Free Software
+## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.  
+
+
+
 ##########################################################################
 ## classes for 2- and higher-dimensional data objects, based on ##########
-## Spatial classes from 'sp'-package                            ##########
+## Spatial classes from 'sp'                           ##########
 
 
 setClass("RFspatialGridDataFrame", contains ="SpatialGridDataFrame",
@@ -53,7 +77,7 @@ setClassUnion("RFspatialDataFrame",
               c("RFspatialGridDataFrame", "RFspatialPointsDataFrame"))
 setClassUnion("RFdataFrame", c("RFgridDataFrame", "RFpointsDataFrame"))
 setClassUnion("RFsp", c("RFspatialGridDataFrame", "RFspatialPointsDataFrame",
-                        "RFgridDataFrame", "RFpointsDataFrame"))
+                       "RFgridDataFrame", "RFpointsDataFrame"))
 
 check.validity.n.vdim <- function(object) {
   if (!all(c("n", "vdim") %in% names(object@.RFparams)))
@@ -97,7 +121,6 @@ setClass('RMmodel',
 ## rules for validity checking of ZF_MODEL objects
 setValidity('RMmodel', 
             function(object){
-              #return(TRUE)
               isRMmodel <- function(x) is(x, class='RMmodel')
               
               isNUMorDEFAULT <- function(x) {
@@ -126,18 +149,24 @@ setValidity('RMmodel',
               }
                                     
               if(!is.null(object@par.general$var) &&
-                 !isRMmodel(object@par.general$var) &&
-                 !is.na(object@par.general$var) &&
-                 !(object@par.general$var==ZF_DEFAULT_STRING))
+                 !isRMmodel(object@par.general$var)) {
+                if (length(object@par.general$var) > 1)
+                  return("not scalar")
+                if (!is.na(object@par.general$var) &&
+                    (object@par.general$var!=ZF_DEFAULT_STRING))
                 if (object@par.general$var < 0)
                   return("negative variance")
+              }
               
               if(!is.null(object@par.general$scale) &&
-                 !isRMmodel(object@par.general$scale) &&
-                 !is.na(object@par.general$scale) &&
-                 !(object@par.general$scale==ZF_DEFAULT_STRING))
+                 !isRMmodel(object@par.general$scale)) {
+                if (length(object@par.general$scale) > 1)
+                 return("not scalar")
+                if (!is.na(object@par.general$scale) &&
+                    object@par.general$scale!=ZF_DEFAULT_STRING)
                 if(object@par.general$scale < 0)
                   return("negative scale")
+              }
               
               if(!is.null(object@par.general$Aniso) &&
                  !isRMmodel(object@par.general$Aniso) &&
@@ -151,14 +180,14 @@ setValidity('RMmodel',
                  !is.na(object@par.general$proj) &&
                  !all(object@par.general$proj==ZF_DEFAULT_STRING)){
                 if(!is.vector(object@par.general$proj) ||
-                   any(object@par.general$proj <= 0) ||
+                   any(object@par.general$proj == 0) ||
+                   any(object@par.general$proj < -2) ||
                    any(object@par.general$proj !=
                        as.integer(object@par.general$proj))
                    )
                   return("proj must be a vector of non-negative integers")
               }
-              
-              return(TRUE)
+             return(TRUE)
             })
 
 
@@ -180,16 +209,19 @@ setClass('RMmodelgenerator', contains ="function",
 
 ## definition of class 'RMmodelFit'
 setClass('RMmodelFit',  contains='RMmodel',
-         representation(likelihood = "numeric",
-                        variab = "matrix",
-                        param = "matrix",
-                        covariab = "ANY",
-                        AIC = "numeric",
-                        AICc = "numeric",
-                        BIC = "numeric",
-                        trend = "numeric",
-                        residuals = "ANY"
-                        )
+         representation(
+             formel = "ANY",
+             likelihood = "numeric",
+             variab = "matrix",
+             param = "matrix",
+             globalvariance  = "ANY",
+             covariat = "ANY",
+             hessian = "ANY",
+             AIC = "numeric",
+             AICc = "numeric",
+             BIC = "numeric",
+             residuals = "ANY"
+             )
          )
 
 
@@ -204,9 +236,10 @@ setClass("RFempVariog",
                         theta.centers = "ANY",
                         T = "ANY",
                         vdim = "ANY",
-                        coord.units = "character",
-                        variab.units = "character",
-                        call = "ANY"
+                        coordunits = "character",
+                        varunits = "character",
+                        call = "ANY",
+                        method = "ANY"
                         )
          )
 
@@ -224,19 +257,18 @@ setValidity("RFempVariog",
 ## definition of class 'RFfit'
 setClass("RFfit", 
          representation(Z = "list",
-                        ev="list",
+                        ev="RFempVariog",
                         table = "data.frame",
                         n.variab = "integer",
                         n.param = "integer",
                         n.covariates = "integer",
-                        boxcox = "logical",
                         deleted = "integer",
                         lowerbounds ='RMmodel',
                         upperbounds ='RMmodel',
                         transform = "list",
                         #vario = "character",
-                        coord.units = "character",
-                        variab.units = "character",
+                        coordunits = "character",
+                        varunits = "character",
                         number.of.data = "integer",
                         modelinfo = "data.frame",
                         number.of.parameters = "integer",
@@ -263,5 +295,4 @@ setClass("RFfit",
          )
 
 ## generic S4 method for 'plot'
-
 setGeneric("plot")
