@@ -1,10 +1,10 @@
- /* 
+/* 
  Authors
  Martin Schlather, schlather@math.uni-mannheim.de
 
  simulation of max-stable random fields
 
- Copyright (C) 2001 -- 2014 Martin Schlather, 
+ Copyright (C) 2001 -- 2017 Martin Schlather, 
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -22,10 +22,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 
-#include <math.h>
+#include <Rmath.h>
 #include <stdio.h>
 #include "RF.h"
-#include "Covariance.h"
+#include "Operator.h"
 
  
 #define POISSON_INTENSITY 0
@@ -35,17 +35,17 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #define PGS_VOXEL 0
 #define PGS_CORNER 1
-#define PGS_SIDES PGS_CORNER + 1
+#define PGS_SIDES (PGS_CORNER + 1)
 
 #define PGS_2D_SIDE1 PGS_SIDES
-#define PGS_2D_SIDE2 PGS_SIDES + 1
+#define PGS_2D_SIDE2 (PGS_SIDES + 1)
 
 #define PGS_3D_SIDE1 PGS_SIDES
-#define PGS_3D_SIDE2 PGS_SIDES + 1
-#define PGS_3D_SIDE3 PGS_SIDES + 2
-#define PGS_3D_EDGE1 PGS_SIDES + 3
-#define PGS_3D_EDGE2 PGS_SIDES + 4
-#define PGS_3D_EDGE3 PGS_SIDES + 5
+#define PGS_3D_SIDE2 (PGS_SIDES + 1)
+#define PGS_3D_SIDE3 (PGS_SIDES + 2)
+#define PGS_3D_EDGE1 (PGS_SIDES + 3)
+#define PGS_3D_EDGE2 (PGS_SIDES + 4)
+#define PGS_3D_EDGE3 (PGS_SIDES + 5)
 
 
 int addUnifModel(cov_model *cov, double radius, cov_model **newmodel) {
@@ -62,16 +62,11 @@ int addUnifModel(cov_model *cov, double radius, cov_model **newmodel) {
 void pts_given_shape(double *x, cov_model *cov, double *v) { 
   cov_model *shape = cov->sub[PGS_FCT];
   NONSTATCOV(x, cov->q, shape, v);
-
-  //printf("x = %f %f %v\n", *x, cov->q[0], *v);
-
 }
 
-void logpts_given_shape(double *x, cov_model *cov, double *v, double *sign) { 
+void logpts_given_shape(double *x, cov_model *cov, double *v, double *Sign) { 
   cov_model *shape = cov->sub[PGS_FCT];
-  LOGNONSTATCOV(x, cov->q, shape, v, sign);
-
-  //  printf("x = %f %f log=%f\n", *x, cov->q[0], *v, *sign);
+  LOGNONSTATCOV(x, cov->q, shape, v, Sign);
 }
 
 int check_pts_given_shape(cov_model *cov) {
@@ -81,6 +76,7 @@ int check_pts_given_shape(cov_model *cov) {
   int err, role,
     dim = cov->tsdim; 
 
+  ASSERT_CARTESIAN;
   if (loc->Time) SERR("Time component not allowed yet"); // todo
    
   kdefault(cov, PGS_RATIO, GLOBAL.extreme.density_ratio); 
@@ -89,17 +85,9 @@ int check_pts_given_shape(cov_model *cov) {
   kdefault(cov, PGS_NORMED, true);
   kdefault(cov, PGS_ISOTROPIC, true);
 
-
-  //PMI(cov, "checking");
- 
   if ((err = checkkappas(cov)) != NOERROR) return err;
 
-  if (cov->q == NULL) {
-    // zwingend CALLOC !
-    if ((cov->q  = (double*) CALLOC(sizeof(double), dim)) == NULL)
-      return ERRORMEMORYALLOCATION;
-    cov->qlen = dim;
-  }
+  if (cov->q == NULL) QALLOC(dim);
   
   if (cov->xdimprev != cov->xdimown || cov->xdimprev != cov->tsdim) 
     return ERRORDIM;
@@ -117,8 +105,6 @@ int check_pts_given_shape(cov_model *cov) {
   } else ILLEGAL_ROLE;
 
 
-  //    printf("here %d\n");
-  
   // isotropy is the property -- user should not define unisotropic functions!
   // so !isotropic only for special, internal definitions, e.g. 
   // strokorbPoly
@@ -127,34 +113,23 @@ int check_pts_given_shape(cov_model *cov) {
   //    err2 = CHECK(shape, dim, dim, ShapeType, XONLY, ISOTROPIC,
   //		 SCALAR, role);
 
-  //  printf("here2\n");
-  // PMI(shape);
- // but it must be treated in any case as if it was non-isotropic
+
+  // but it must be treated in any case as if it was non-isotropic
   if ((err = CHECK(shape, dim, dim, ShapeType, XONLY, CARTESIAN_COORD,
 		   SCALAR, role)) != NOERROR) {
     if (P0INT(PGS_ISOTROPIC)) BUG;
     XERR(err);
     return err;
   }
-  // if (err2 != NOERROR) SERR("isotropic shape function expected.");
 
   setbackward(cov, shape);
 
   //if (!shape->deterministic) return ERRORNOTPROGRAMMED; // Dichte muss nicht normiert sein und stattdessen durch das mittlere Volumen dividiert werden.
 
   if (pts != NULL) {
-
-    //printf("pts %s %s\n", CovList[pts->nr + 1].nick, CovList[pts->nr + 1].name);
-    //PMI(pts);
-
-    //      assert(dim == 2); print("x");
-    // printf("dim=%d\n", dim);
-    // APMI(pts);
-
     if ((err = CHECK_R(pts, dim)) != NOERROR) return err;
   }
 
-  //  printf("done\n");
  
   return NOERROR;
 }
@@ -167,7 +142,7 @@ int struct_pts_given_shape(cov_model *cov, cov_model **newmodel){
   int err = NOERROR;
 
   ASSERT_NEWMODEL_NULL;
-  if (cov->Spgs != NULL)  PGS_DELETE(&(cov->Spgs));
+  if (cov->Spgs != NULL)  pgs_DELETE(&(cov->Spgs));
 
   if (shape->role != ROLE_POISSON && shape->role != ROLE_MAXSTABLE)
     ILLEGAL_ROLE;
@@ -253,7 +228,7 @@ int calculate_mass_gauss(cov_model *cov) {
 
   
     // is the underlying point distribution uniform? Then
-    // no safety division by sqrt(dim) seems to be necessary // todo
+    // no safety division by SQRT(dim) seems to be necessary // todo
     // unif might be modified by $. So a check on the equality
     // of three values is done
     VTLG_D(ZERO, pts, v);
@@ -269,7 +244,7 @@ int calculate_mass_gauss(cov_model *cov) {
     }
     if (zx != v[0] || zy != v[0] ||  true) {
       for (d=0; d<dim; d++) {
-	y[d] /= sqrt((double) dim); // falls nicht runif
+	y[d] /= SQRT((double) dim); // falls nicht runif
       }
     }
     
@@ -281,10 +256,10 @@ int calculate_mass_gauss(cov_model *cov) {
     for (d=0; d<dim; d++) { // Berechnung der ausgeduennten Reihe
       if (loc->xgr[d][XLENGTH] > 1) {
 	double range = loc->xgr[d][XSTEP] * (loc->xgr[d][XLENGTH] - 1.0);
-	xgr[d][XLENGTH] = ceil(range / y[d] + 1.0);
+	xgr[d][XLENGTH] = CEIL(range / y[d] + 1.0);
 	if (xgr[d][XLENGTH] >= loc->xgr[d][XLENGTH]) {
 	  BUG;
-	  memcpy(xgr[d], loc->xgr[d], 3 * sizeof(double));
+	  MEMCOPYX(xgr[d], loc->xgr[d], 3 * sizeof(double));
 	} else {
 	  xgr[d][XSTART] =
 	    loc->xgr[d][XSTART] - 0.5 *((xgr[d][XLENGTH] - 1.0) * y[d] - range);
@@ -483,21 +458,21 @@ int complete_copy(cov_model **newmodel, cov_model *cov) {
   prev = start->key != NULL ? start->key : start->sub[0];
   if (prev->typus != ProcessType) BUG;
   
-  if ((err = covcpy(newmodel, prev)) != NOERROR) return err;
+  if ((err = covCpy(newmodel, prev)) != NOERROR) return err;
   (*newmodel)->calling = cov;
   int role = prev->role ;// role_of_process(prev->nr);
   if ((err = CHECK(*newmodel, prev->tsdim, prev->xdimprev, prev->typus, 
-		   prev->domprev, prev->isoprev, prev->vdim2, role)) 
+		   prev->domprev, prev->isoprev, prev->vdim, role)) 
       != NOERROR)  return err;  
    if ((err = STRUCT(*newmodel, NULL)) != NOERROR) { return err; }
   if (!(*newmodel)->initialised) {
     if ((err =  CHECK(*newmodel, prev->tsdim, prev->xdimprev, prev->typus, 
-		      prev->domprev, prev->isoprev, prev->vdim2, role)) 
+		      prev->domprev, prev->isoprev, prev->vdim, role)) 
 	!= NOERROR) return err;  
  
-    NEW_COV_STORAGE(*newmodel, stor, STORAGE, gen_storage);
+    NEW_COV_STORAGE(*newmodel, gen);
    //APMI(*newmodel);
-    if ((err = INIT(*newmodel, 0, cov->stor)) != NOERROR) {
+    if ((err = INIT(*newmodel, 0, cov->Sgen)) != NOERROR) {
       //APMI(cov); // !!! ?? hier weitermachen
       return err; 
     }
@@ -528,7 +503,7 @@ int init_pts_given_shape(cov_model *cov, gen_storage *S) {
   assert(dim == cov->sub[PGS_LOC]->xdimprev);
   assert(dim == Loc(cov)->timespacedim);
   
-  if (Cshape->inverse == ErrCov)
+  if (Cshape->inverse == ErrCov) // && Cshape->nonstat_inverse == ErrCovNonstat)
     SERR1("support of the model is unknown. Use '%s' to determine the support",
 	  CovList[TRUNCSUPPORT].nick);
  
@@ -584,7 +559,7 @@ int init_pts_given_shape(cov_model *cov, gen_storage *S) {
 	 )) return ERRORMEMORYALLOCATION;
     for (i=3, d=1; d<dim; d++, i+=3) pgs->xgr[d] = pgs->xgr[0] + i;
     // printf("%d\n", pgs->xgr[1] - pgs->xgr[0]); assert(false);
-   if ((err = calculate_mass_gauss(cov)) != NOERROR) return err;
+    if ((err = calculate_mass_gauss(cov)) != NOERROR) return err;
   } else if (hasMaxStableRole(cov)) {
     if (pgsnull &&
 	(
@@ -707,7 +682,7 @@ int DrawCathegory(int size, double *single,
     
     for (i = 0; mass > total[i]; i++);
     if (i > 0) mass -= total[i-1];
-    *elmts = floor(mass / single[i]);
+    *elmts = FLOOR(mass / single[i]);
     //printf("draw: i=%d tot=%f mass=%f sing=%f elm=%d\n", i, total[i], mass, single[i], *elmts);
     return i;
   } else {
@@ -750,7 +725,7 @@ void do_pgs_gauss(cov_model *cov, gen_storage *S) {
     DORANDOM(pts, cov->q);  // cov->q nur dummy. Wird ueberschrieben
     if (cov->role == ROLE_POISSON_GAUSS || !grid) { 
       if (calculate_mass_gauss(cov) != NOERROR) 
-	error("unexpected error in 'do_pts_given_shape' (maxstable)");
+	ERR("unexpected error in 'do_pts_given_shape' (maxstable)");
     } else BUG;
   }
 
@@ -776,7 +751,7 @@ void do_pgs_gauss(cov_model *cov, gen_storage *S) {
       cov->q[d] = xgr[d][XSTART] + xgr[d][XSTEP] * which + v[d];
       // to determine the points that contribute to the value of the 
       // density function at cov->q
-      min[d] = ceil((cov->q[d] - y[d] - xgr[d][XSTART]) / xgr[d][XSTEP]);
+      min[d] = CEIL((cov->q[d] - y[d] - xgr[d][XSTART]) / xgr[d][XSTEP]);
       max[d] = (cov->q[d] - x[d] - xgr[d][XSTART]) / xgr[d][XSTEP];
 
       // printf("%d %d delat=%f %f y=%f %f step=%f\n", min[d], max[d], 
@@ -834,7 +809,7 @@ void do_pgs_gauss(cov_model *cov, gen_storage *S) {
       total += value;
     }
   }
-  pgs->log_density = log(total / pgs->totalmass);
+  pgs->log_density = LOG(total / pgs->totalmass);
 
   //printf("total=%f %f grid=%d dim=%d\n", total, pgs->totalmass,loc->grid,dim);
 
@@ -892,7 +867,7 @@ void do_pgs_maxstable(cov_model *cov, gen_storage *S) {
       if ((err = REINIT(cov, cov->mpp.moments, S)) != NOERROR) BUG;
       DO(shape, S);     
       if (calculate_mass_maxstable(cov) != NOERROR) 
-	error("unexpected error in 'do_pts_given_shape' (maxstable)");
+	ERR("unexpected error in 'do_pts_given_shape' (maxstable)");
 
       cmaxDmu = pgs->totalmass / shape->mpp.mMplus[1];
       if (pgs->n_zhou_c < GLOBAL.extreme.max_n_zhou) {
@@ -943,7 +918,7 @@ void do_pgs_maxstable(cov_model *cov, gen_storage *S) {
     if (prev->key != NULL) prev->key = cov;		
     else if (prev->sub[0] != NULL) prev->sub[0] = cov;	
     else if (prev->sub[1] != NULL) prev->sub[1] = cov;	
-    else error("structure mismatch");			
+    else ERR("structure mismatch");			
 
     // APMI(cov);
 
@@ -952,7 +927,7 @@ void do_pgs_maxstable(cov_model *cov, gen_storage *S) {
     //PMI(pts);
     // printf("%f %f\n", PARAM(pts, 1)[0] *PARAM(pts, 1)[1] * 4 , pts->mpp.mMplus[0]);
     
-    //assert(fabs(PARAM(pts, 1)[0] *PARAM(pts, 1)[1] * 4 - pts->mpp.mMplus[0]) < 1e-14 *  pts->mpp.mMplus[0]);
+    //assert(FABS(PARAM(pts, 1)[0] *PARAM(pts, 1)[1] * 4 - pts->mpp.mMplus[0]) < 1e-14 *  pts->mpp.mMplus[0]);
 
     //   PMI(save_pts);
     //APMI(pts);
@@ -1026,9 +1001,9 @@ void do_pgs_maxstable(cov_model *cov, gen_storage *S) {
     // Bsp das nicht funktioniert fuer flat=FLAT_UNDETERMINED
     //    x _ seq(1, 10, 0.01)
     //      z _ RFsimulate(RPsmith(RMgauss(), xi=0), x, x, grid=T, print=20)
-    //      Print(exp(-exp(range(-z[[1]]))))
+    //      Print(EXP(-EXP(range(-z[[1]]))))
     //      plot(z); X11(); hist(z[[1]], freq=FALSE, 50)
-    //			curve(exp(-x-2) * exp(-exp(-x-2)), -5, 2, add=TRUE)
+    //			curve(EXP(-x-2) * EXP(-EXP(-x-2)), -5, 2, add=TRUE)
     if (i != PGS_VOXEL) VTLG_R(x, pts, v); 
     //  printf("v=%f x=%f %d %d %d \n", v[0],  x[0], i, PGS_VOXEL,  PGS_CORNER);
     for (d = 0; d<dim; d++)
@@ -1096,7 +1071,7 @@ void do_pgs_maxstable(cov_model *cov, gen_storage *S) {
     //printf("log density v=%f, %f, %f %e\n", v[0],v[1],v[2], pgs->log_density);
 
     //printf("log_dens %f\n", pgs->log_density);
-    //  pgs->log_density -= log(pgs->totalmass); // ueberprueft: normierung
+    //  pgs->log_density -= LOG(pgs->totalmass); // ueberprueft: normierung
     // zu einer WK'Dichte.
   } else {
     double density;
@@ -1105,7 +1080,7 @@ void do_pgs_maxstable(cov_model *cov, gen_storage *S) {
     //  printf("dens %f\n", density);
       //  printf("density %e v=%f %f %f\n", density, v[0], v[1], v[2]);
 
-    pgs->log_density = log(density);
+    pgs->log_density = LOG(density);
   }
 
  
@@ -1131,12 +1106,14 @@ void do_pts_given_shape(cov_model *cov, gen_storage *S) {
   } else {
     PMI(cov); BUG; //
   }
+
+  // do_gs_maxstable might break links of the cov structure:
   cov_model *prev=cov->calling;		
   assert(prev != NULL);				
   if (prev->key != NULL) cov = prev->key;		
   else if (prev->sub[0] != NULL) cov = prev->sub[0];	
   else if (prev->sub[1] != NULL) cov = prev->sub[1];	
-  else error("structure mismatch");			
+  else ERR("structure mismatch");			
   
   cov_model *shape = cov->sub[PGS_FCT],
     *pts = cov->sub[PGS_LOC];
@@ -1149,7 +1126,7 @@ void do_pts_given_shape(cov_model *cov, gen_storage *S) {
     *y = pgs->y;
 
   if (cov->role == ROLE_POISSON_GAUSS) {
-    eps = GLOBAL.mpp.about_zero * exp(pgs->log_density);
+    eps = GLOBAL.mpp.about_zero * EXP(pgs->log_density);
   } else if (hasMaxStableRole(cov)) { // todo: sauber trennen, wann 
     // max-stable, smith etc
     eps = pgs->currentthreshold;
@@ -1161,7 +1138,7 @@ void do_pts_given_shape(cov_model *cov, gen_storage *S) {
       BUG;
     }
     if (cov->loggiven)  eps += pgs->log_density;
-    else eps *= exp(pgs->log_density);
+    else eps *= EXP(pgs->log_density);
   } else BUG;
 
   
@@ -1188,7 +1165,7 @@ void do_pts_given_shape(cov_model *cov, gen_storage *S) {
     if (ISNAN(x[0])  || x[0] > y[0]) BUG;
  
     //printf("eps=%f %f %f x=(%e %e %e) y=(%e %e %e) q=(%f %f %f)\n",
-    //     eps, pgs->currentthreshold,  exp(pgs->log_density),
+    //     eps, pgs->currentthreshold,  EXP(pgs->log_density),
     //     x[0], x[1], x[2], y[0], y[1], y[2],
     //     cov->q[0], cov->q[1], cov->q[2]);
   }
@@ -1203,7 +1180,7 @@ void do_pts_given_shape(cov_model *cov, gen_storage *S) {
     pgs->supportmin[d] = cov->q[d] - y[d]; // 4 * for debugging...
     pgs->supportmax[d] = cov->q[d] - x[d];
 
-     //     printf("do d=%d q=%f x=%f y=%f eps=%e thr=%e %f %f\n",
+    //     printf("do d=%d q=%f x=%f y=%f eps=%e thr=%e %f %f\n",
     //	  	  d, cov->q[d], x[d], y[d], eps, pgs->currentthreshold, pgs->supportmin[d], pgs->supportmax[d]);
 
   if (ISNAN(pgs->supportmin[d]) || ISNAN(pgs->supportmax[d]) ||
@@ -1220,7 +1197,6 @@ void do_pts_given_shape(cov_model *cov, gen_storage *S) {
   
 
   //  BUG;
-
   //   APMI(cov);
 
   cov->fieldreturn = shape->fieldreturn;
@@ -1267,14 +1243,144 @@ void range_pts_given_shape(cov_model VARIABLE_IS_NOT_USED *cov, range_type *rang
 
 
 
+
+
+#define STAT_SHAPE_FCT 0
+void stationary_shape(double *x, cov_model *cov, double *v) { 
+  cov_model *shape = cov->sub[STAT_SHAPE_FCT];
+  FCTN(x, shape, v);
+  //  if (cov->q[0] < 1) {
+  //   printf("x=%f q=%f v=%f\n", x[0], cov->q[0], v[0]);
+  //   assert(false);
+  // } 
+  //  APMI(cov);
+}
+
+void logstationary_shape(double *x, cov_model *cov, double *v, double *Sign) { 
+  cov_model *shape = cov->sub[STAT_SHAPE_FCT];
+  LOGCOV(x, shape, v, Sign);
+}
+
+int check_stationary_shape(cov_model *cov) {
+  cov_model *shape = cov->sub[STAT_SHAPE_FCT];
+  int err, role,
+    dim = cov->tsdim; 
+
+   ASSERT_CARTESIAN;
+ 
+  if (cov->xdimprev != cov->xdimown || cov->xdimprev != cov->tsdim) 
+    return ERRORDIM;
+  
+  if (cov->role == ROLE_GAUSS) {
+    role = isGaussProcess(shape) ? ROLE_GAUSS
+      : shape->nr == BINARYPROC ? ROLE_GAUSS
+      : ROLE_UNDEFINED; 
+    ASSERT_ROLE_DEFINED(shape);
+  } else if (hasPoissonRole(cov)) {
+    role = ROLE_POISSON;
+  } else if (hasMaxStableRole(cov)) {
+    role = ROLE_MAXSTABLE;
+  } else ILLEGAL_ROLE;
+
+
+  //printf("here\n");
+  
+  if ((err = CHECK(shape, dim, dim, ProcessType, XONLY, CARTESIAN_COORD, 
+		     SCALAR, role)) != NOERROR)  return err;
+  setbackward(cov, shape);
+
+  return NOERROR;
+}
+
+int struct_stationary_shape(cov_model *cov, cov_model **newmodel){
+  cov_model *shape = cov->sub[STAT_SHAPE_FCT];
+  //  location_type *loc = Loc(cov);
+
+  ASSERT_NEWMODEL_NULL;
+
+  if (shape->role != ROLE_POISSON && shape->role != ROLE_MAXSTABLE)
+    ILLEGAL_ROLE;
+
+  //printf("here\n");
+
+  return NOERROR;
+}
+
+
+int init_stationary_shape(cov_model *cov, gen_storage *S) {  
+  cov_model *shape = cov->sub[STAT_SHAPE_FCT];
+  int d, i,
+    err = NOERROR,
+    dim = shape->xdimprev;
+ 
+  //PMI(cov);
+
+  assert(dim == Loc(cov)->timespacedim);
+  if ((err = alloc_pgs(cov)) != NOERROR) return err;
+  pgs_storage *pgs = cov->Spgs;
+
+  // selbst wenn zufaelliger Shape: 1x laufen lassen, ob 
+  // Fehler auftauchen. Unter "Do" lassen sie sich nicht mehr
+  // schoen abfangen.
+   
+  assert(cov->mpp.moments >= 1);
+  if ((err = INIT(shape, 1, S)) != NOERROR) {
+    return err; //gatter?
+  }
+  assert(shape->mpp.moments >= 1);
+  for (i=0; i<=cov->mpp.moments; i++) {
+    cov->mpp.mM[i] = shape->mpp.mM[i];
+    cov->mpp.mMplus[i] = shape->mpp.mMplus[i];
+  }
+
+   
+  pgs->zhou_c = 1.0 / cov->mpp.mMplus[1]; // passt fuer binary, und auch fuer 
+  if (!R_FINITE(pgs->zhou_c))
+    SERR1("max height of '%s' not finite", NICK(shape));
+  pgs->estimated_zhou_c = false; 
+  if (!cov->deterministic) SERR("not deterministic shapes in stationary modelling -- please contact author");
+  
+  pgs->log_density = 0; 
+   
+  //  printf("dims %d %d\n", cov->xdimown, shape->xdimprev);
+
+  for (d=0; d<dim; d++) {
+    pgs->supportmin[d] = RF_NEGINF; // 4 * for debugging...
+    pgs->supportmax[d] = RF_INF;
+  }
+
+  cov->mpp.maxheights[0] = shape->mpp.maxheights[0];
+  cov->rf = shape->rf;
+  cov->origrf = false;
+  cov->fieldreturn = shape->fieldreturn;
+  if (!cov->fieldreturn) BUG;
+
+
+  // APMI(cov);
+
+  return NOERROR;
+}
+
+
+void do_stationary_shape(cov_model *cov, gen_storage *S) {
+  cov_model *shape = cov->sub[STAT_SHAPE_FCT]; 
+  DO(shape, S);
+  cov->mpp.maxheights[0] = shape->mpp.maxheights[0];
+  assert(shape->fieldreturn);
+}
+
+
+
+ 
+
 void standard_shape(double *x, cov_model *cov, double *v) { 
   cov_model *shape = cov->sub[PGS_FCT];
   NONSTATCOV(x, cov->q, shape, v);
 }
 
-void logstandard_shape(double *x, cov_model *cov, double *v, double *sign) { 
+void logstandard_shape(double *x, cov_model *cov, double *v, double *Sign) { 
   cov_model *shape = cov->sub[PGS_FCT];
-  LOGNONSTATCOV(x, cov->q, shape, v, sign);
+  LOGNONSTATCOV(x, cov->q, shape, v, Sign);
 }
 
 int check_standard_shape(cov_model *cov) {
@@ -1283,13 +1389,9 @@ int check_standard_shape(cov_model *cov) {
   int err, role,
     dim = cov->tsdim; 
   
+  ASSERT_CARTESIAN;
 
-  if (cov->q == NULL) {
-    // zwingend CALLOC !
-    if ((cov->q  = (double*) CALLOC(sizeof(double), dim)) == NULL)
-      return ERRORMEMORYALLOCATION;
-    cov->qlen = dim;
-  }
+  if (cov->q == NULL) QALLOC(dim);
 
   if (cov->xdimprev != cov->xdimown || cov->xdimprev != cov->tsdim) 
     return ERRORDIM;
@@ -1305,7 +1407,8 @@ int check_standard_shape(cov_model *cov) {
 		     SCALAR, role)) != NOERROR)  return err;
   setbackward(cov, shape);
   
-  if (!shape->deterministic) SERR("random shapes for 'standard' not allowed yet");
+  if (!shape->deterministic) 
+    SERR1("random shapes for '%s' not allowed yet", NICK(cov));
 
   if (pts != NULL) {
     if ((err = CHECK_R(pts, dim)) != NOERROR) return err;
@@ -1314,6 +1417,8 @@ int check_standard_shape(cov_model *cov) {
   return NOERROR;
 }
 
+
+
 int struct_standard_shape(cov_model *cov, cov_model **newmodel){
   cov_model *shape = cov->sub[PGS_FCT];
   //  int    err = NOERROR;
@@ -1337,6 +1442,8 @@ int struct_standard_shape(cov_model *cov, cov_model **newmodel){
 }
  
 
+
+
 int init_standard_shape(cov_model *cov, gen_storage *S) {  
   cov_model *shape = cov->sub[PGS_FCT];
   //  location_type *loc = Loc(cov);
@@ -1370,8 +1477,6 @@ int init_standard_shape(cov_model *cov, gen_storage *S) {
   // schoen abfangen.
   // ROLE_ASSERT(ROLE_POISSON || cov->role == ROLE_MAXSTABLE);
 
-  //PMI(cov->calling, -1);
-  
   assert(cov->mpp.moments == hasMaxStableRole(cov));
   if ((err = INIT(shape, cov->mpp.moments, S)) != NOERROR) return err; //gatter?
 
@@ -1391,11 +1496,14 @@ int init_standard_shape(cov_model *cov, gen_storage *S) {
   GetDiameter(loc, pgs->localmin, pgs->localmax, 
 	      pgs->supportcentre); // last is only a dummy
   pgs->totalmass = 1.0;
+
+  // PMI(cov); printf("d=%d", dim);
+  
   for (d=0; d<dim; d++) {
     min[d] = pgs->localmin[d] - y[d];
     max[d] = pgs->localmax[d] - x[d];
     if (!(R_FINITE(min[d]) && R_FINITE(max[d])))
-      SERR("simulation window does not have compact support. Should 'RMtruncsupport' be used?");
+      SERR1("simulation window does not have compact support. Should '%s' be used?", CovList[TRUNCSUPPORT].nick);
     pgs->totalmass *= max[d] - min[d];
   }
   
@@ -1471,32 +1579,44 @@ void do_standard_shape(cov_model *cov, gen_storage *S) {
 
 
 
-#define STAT_SHAPE_FCT 0
-void stationary_shape(double *x, cov_model *cov, double *v) { 
-  cov_model *shape = cov->sub[STAT_SHAPE_FCT];
-  FCTN(x, shape, v);
-  //  if (cov->q[0] < 1) {
-  //   printf("x=%f q=%f v=%f\n", x[0], cov->q[0], v[0]);
-  //   assert(false);
-  // } 
-  //  APMI(cov);
+void mcmc_pgs(double VARIABLE_IS_NOT_USED *x, cov_model VARIABLE_IS_NOT_USED *cov, double VARIABLE_IS_NOT_USED *v) { 
 }
 
-void logstationary_shape(double *x, cov_model *cov, double *v, double *sign) { 
-  cov_model *shape = cov->sub[STAT_SHAPE_FCT];
-  LOGCOV(x, shape, v, sign);
-}
+void logmcmc_pgs(double *x, cov_model *cov, double *v, double *Sign) { 
+  cov_model *shape = cov->sub[PGS_FCT];
+  LOGNONSTATCOV(x, cov->q, shape, v, Sign);
 
-int check_stationary_shape(cov_model *cov) {
-  cov_model *shape = cov->sub[STAT_SHAPE_FCT];
+  //  printf("x = %f %f log=%f\n", *x, cov->q[0], *v, *Sign);
+}
+int check_mcmc_pgs(cov_model *cov) {
+  cov_model *shape = cov->sub[PGS_FCT],
+    *pts = cov->sub[PGS_LOC];
+  location_type *loc = Loc(cov);
   int err, role,
     dim = cov->tsdim; 
+
+  ASSERT_CARTESIAN;
+  if (loc->Time) SERR("Time component not allowed yet"); // todo
+   
+  kdefault(cov, PGS_RATIO, GLOBAL.extreme.density_ratio); 
+  kdefault(cov, PGS_FLAT, GLOBAL.extreme.flat);
+  kdefault(cov, PGS_INFTY_SMALL, !PINT(PGS_FLAT));
+  kdefault(cov, PGS_NORMED, true);
+  kdefault(cov, PGS_ISOTROPIC, true);
+
+
+  //PMI(cov, "checking");
+ 
+  if ((err = checkkappas(cov)) != NOERROR) return err;
+
+  if (cov->q == NULL) QALLOC(dim);
   
   if (cov->xdimprev != cov->xdimown || cov->xdimprev != cov->tsdim) 
     return ERRORDIM;
   
   if (cov->role == ROLE_GAUSS) {
-    role = isGaussProcess(shape) ? ROLE_GAUSS
+    role = isShape(shape) ? cov->role  //Marco, 29.5.13
+      : isGaussProcess(shape) ? ROLE_GAUSS
       : shape->nr == BINARYPROC ? ROLE_GAUSS
       : ROLE_UNDEFINED; 
     ASSERT_ROLE_DEFINED(shape);
@@ -1507,89 +1627,131 @@ int check_stationary_shape(cov_model *cov) {
   } else ILLEGAL_ROLE;
 
 
-  //printf("here\n");
+  //    printf("here %d\n");
   
-  if ((err = CHECK(shape, dim, dim, ProcessType, XONLY, CARTESIAN_COORD, 
-		     SCALAR, role)) != NOERROR)  return err;
+  // isotropy is the property -- user should not define unisotropic functions!
+  // so !isotropic only for special, internal definitions, e.g. 
+  // strokorbPoly
+  // int err2 = NOERROR;
+  //  if (P0INT(PGS_ISOTROPIC)) 
+  //    err2 = CHECK(shape, dim, dim, ShapeType, XONLY, ISOTROPIC,
+  //		 SCALAR, role);
+
+  //  printf("here2\n");
+  // PMI(shape);
+ // but it must be treated in any case as if it was non-isotropic
+  if ((err = CHECK(shape, dim, dim, ShapeType, XONLY, CARTESIAN_COORD,
+		   SCALAR, role)) != NOERROR) {
+    if (P0INT(PGS_ISOTROPIC)) BUG;
+    XERR(err);
+    return err;
+  }
+  // if (err2 != NOERROR) SERR("isotropic shape function expected.");
+
   setbackward(cov, shape);
 
-  return NOERROR;
-}
+  //if (!shape->deterministic) return ERRORNOTPROGRAMMED; // Dichte muss nicht normiert sein und stattdessen durch das mittlere Volumen dividiert werden.
 
-int struct_stationary_shape(cov_model *cov, cov_model **newmodel){
-  cov_model *shape = cov->sub[STAT_SHAPE_FCT];
-  //  location_type *loc = Loc(cov);
+  if (pts != NULL) {
 
-  ASSERT_NEWMODEL_NULL;
+    //printf("pts %s %s\n", CovList[pts->nr + 1].nick, CovList[pts->nr + 1].name);
+    //PMI(pts);
 
-  if (shape->role != ROLE_POISSON && shape->role != ROLE_MAXSTABLE)
-    ILLEGAL_ROLE;
+    //      assert(dim == 2); print("x");
+    // printf("dim=%d\n", dim);
+    // APMI(pts);
 
-  //printf("here\n");
+    if ((err = CHECK_R(pts, dim)) != NOERROR) return err;
+  }
 
-  return NOERROR;
-}
-
-
-int init_stationary_shape(cov_model *cov, gen_storage *S) {  
-  cov_model *shape = cov->sub[STAT_SHAPE_FCT];
-  int d, i,
-    err = NOERROR,
-    dim = shape->xdimprev;
+  //  printf("done\n");
  
+  return NOERROR;
+}
+
+
+
+int struct_mcmc_pgs(cov_model VARIABLE_IS_NOT_USED *cov, cov_model VARIABLE_IS_NOT_USED **newmodel){
+  //  cov_model *shape = cov->sub[PGS_FCT];
+  //  location_type *loc = Loc(cov);
+  //  int err = NOERROR;
+  ASSERT_NEWMODEL_NULL;
+ return NOERROR;
+}
+
+
+int init_mcmc_pgs(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S) {  
+  cov_model *shape = cov->sub[PGS_FCT],
+    *pts = cov->sub[PGS_LOC];
+  //  cov_fct *Cshape = CovList + shape->nr;
+  //location_type *loc = Loc(cov);
+  int i,
+    //dim = shape->xdimprev,
+    err = NOERROR;
+  // pgs_storage *pgs = cov->Spgs;
+  //  bool 
+  //   grid = Loc(cov)->grid,
+  //  pgsnull = pgs == NULL;
   //PMI(cov);
 
-  assert(dim == Loc(cov)->timespacedim);
-  if ((err = alloc_pgs(cov)) != NOERROR) return err;
-  pgs_storage *pgs = cov->Spgs;
-
-  // selbst wenn zufaelliger Shape: 1x laufen lassen, ob 
-  // Fehler auftauchen. Unter "Do" lassen sie sich nicht mehr
-  // schoen abfangen.
-
-  
-  assert(cov->mpp.moments >= 1);
-  if ((err = INIT(shape, 1, S)) != NOERROR) return err; //gatter?
-  assert(shape->mpp.moments >= 1);
   for (i=0; i<=cov->mpp.moments; i++) {
-    cov->mpp.mM[i] = shape->mpp.mM[i];
-    cov->mpp.mMplus[i] = shape->mpp.mMplus[i];
+    //    printf("%d %f %f %d\n", i, pts->mpp.mM[i], shape->mpp.mMplus[i], cov->mpp.moments);
+
+    //assert(pts->mpp.mMplus[0] == 1.0);
+    
+    cov->mpp.mM[i] = shape->mpp.mM[i] * pts->mpp.mMplus[0];
+    cov->mpp.mMplus[i] = shape->mpp.mMplus[i] * pts->mpp.mMplus[0];
   }
 
-   
-  pgs->zhou_c = 1.0 / cov->mpp.mMplus[1]; // passt fuer binary, und auch fuer 
-  if (!R_FINITE(pgs->zhou_c))
-    SERR1("max height of '%s' not finite", NICK(shape));
-  pgs->estimated_zhou_c = false; 
-  if (!cov->deterministic) SERR("not deterministic shapes in stationary modelling -- please contact author");
   
-  pgs->log_density = 0; 
    
-  //  printf("dims %d %d\n", cov->xdimown, shape->xdimprev);
-
-  for (d=0; d<dim; d++) {
-    pgs->supportmin[d] = RF_NEGINF; // 4 * for debugging...
-    pgs->supportmax[d] = RF_INF;
-  }
-
-  cov->mpp.maxheights[0] = shape->mpp.maxheights[0];
   cov->rf = shape->rf;
   cov->origrf = false;
-  cov->fieldreturn = shape->fieldreturn;
-  if (!cov->fieldreturn) BUG;
-
-
+ 
   // APMI(cov);
 
-  return NOERROR;
+  return err;
 }
 
 
-void do_stationary_shape(cov_model *cov, gen_storage *S) {
-  cov_model *shape = cov->sub[STAT_SHAPE_FCT]; 
-  DO(shape, S);
-  cov->mpp.maxheights[0] = shape->mpp.maxheights[0];
-  assert(shape->fieldreturn);
+void do_mcmc_pgs(cov_model  VARIABLE_IS_NOT_USED *cov, gen_storage  VARIABLE_IS_NOT_USED *S) {
 }
 
+void range_mcmc_pgs(cov_model VARIABLE_IS_NOT_USED *cov, range_type *range) {
+  
+  range->min[PGS_RATIO] = 0;
+  range->max[PGS_RATIO] = 1;
+  range->pmin[PGS_RATIO] = 0;
+  range->pmax[PGS_RATIO] = 1;
+  range->openmin[PGS_RATIO] = false;
+  range->openmax[PGS_RATIO] = false; 
+
+  range->min[PGS_FLAT] = -1;
+  range->max[PGS_FLAT] = 1;
+  range->pmin[PGS_FLAT] = -1;
+  range->pmax[PGS_FLAT] = 1;
+  range->openmin[PGS_FLAT] = false;
+  range->openmax[PGS_FLAT] = false; 
+
+  range->min[PGS_INFTY_SMALL] = 0;
+  range->max[PGS_INFTY_SMALL] = 1;
+  range->pmin[PGS_INFTY_SMALL] = 0;
+  range->pmax[PGS_INFTY_SMALL] = 1;
+  range->openmin[PGS_INFTY_SMALL] = false;
+  range->openmax[PGS_INFTY_SMALL] = false; 
+
+  range->min[PGS_NORMED] = 0;
+  range->max[PGS_NORMED] = 1;
+  range->pmin[PGS_NORMED] = 0;
+  range->pmax[PGS_NORMED] = 1;
+  range->openmin[PGS_NORMED] = false;
+  range->openmax[PGS_NORMED] = false; 
+
+  range->min[PGS_ISOTROPIC] = 0;
+  range->max[PGS_ISOTROPIC] = 1;
+  range->pmin[PGS_ISOTROPIC] = 0;
+  range->pmax[PGS_ISOTROPIC] = 1;
+  range->openmin[PGS_ISOTROPIC] = false;
+  range->openmax[PGS_ISOTROPIC] = false; 
+}
 
