@@ -1,7 +1,35 @@
+
+
+
+/*
+ Authors 
+ Martin Schlather, schlather@math.uni-mannheim.de
+
+
+ Copyright (C) 2015 -- 2017 Martin Schlather
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 3
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.  
+*/
+
+
+
 #ifndef Primitives_H
 #define Primitives_H 1
 
 #define UNIT_EPSILON 1E-13
+#include "Coordinate_systems.h"
 
 
 void AngleMatrix(cov_model *cov, double *A);
@@ -21,7 +49,7 @@ void rangeBessel(cov_model *cov, range_type *ra);
 
 /* Cauchy models */
 void Cauchy(double *x, cov_model *cov, double *v);
-void logCauchy(double *x, cov_model *cov, double *v, double *sign);
+void logCauchy(double *x, cov_model *cov, double *v, double *Sign);
 void TBM2Cauchy(double *x, cov_model *cov, double *v);
 void DCauchy(double *x, cov_model *cov, double *v);
 void DDCauchy(double *x, cov_model *cov, double *v);
@@ -46,14 +74,26 @@ void Dcircular(double *x, cov_model *cov, double *v);
 int structcircular(cov_model *cov,  cov_model **);
 
 
+
+void kappaconstant(int i, cov_model *cov, int *nr, int *nc);
 void constant(double *x, cov_model *cov, double *v);
-void constant_nonstat(double *x, double *y, cov_model *cov, double *v);
-void covmatrix_constant(cov_model *cov, double *v);
-char iscovmatrix_constant(cov_model *cov);
+void nonstatconstant(double *x, double *y, cov_model *cov, double *v);
 void rangeconstant(cov_model *cov, range_type* ra);
 int checkconstant(cov_model *cov) ;
 
 
+void covariate(double *x, cov_model *cov, double *v);
+int checkcovariate(cov_model *cov);
+void rangecovariate(cov_model VARIABLE_IS_NOT_USED *cov, range_type *range);
+void kappa_covariate(int i, cov_model *cov, int *nr, int *nc);
+
+void fixStat(double *x, cov_model *cov, double *v);
+void fix(double *x, double *y, cov_model *cov, double *v);
+int checkfix(cov_model *cov);
+void rangefix(cov_model VARIABLE_IS_NOT_USED *cov, range_type *range);
+void kappa_fix(int i, cov_model *cov, int *nr, int *nc);
+
+
 /* coxgauss, cmp with nsst1 !! */
 // see NonIsoCovFct.cc
 
@@ -71,11 +111,13 @@ void dagum(double *x, cov_model *cov, double *v);
 void Inversedagum(double *x, cov_model *cov, double *v); 
 void Ddagum(double *x, cov_model *cov, double *v);
 void rangedagum(cov_model *cov, range_type* ra);
- 
+int checkdagum(cov_model *cov);
+int initdagum(cov_model *cov, gen_storage *stor);
+
 
 /*  damped cosine -- derivative of exponential:*/
 void dampedcosine(double *x, cov_model *cov, double *v);
-void logdampedcosine(double *x, cov_model *cov, double *v, double *sign);
+void logdampedcosine(double *x, cov_model *cov, double *v, double *Sign);
 void Inversedampedcosine(double *x, cov_model *cov, double *v); 
 void Ddampedcosine(double *x, cov_model *cov, double *v);
 void rangedampedcosine(cov_model *cov, range_type* ra);
@@ -86,9 +128,12 @@ int checkdampedcosine(cov_model *cov);
 void dewijsian(double *x, cov_model *cov, double *v);
 void Ddewijsian(double *x, cov_model *cov, double *v);
 void DDdewijsian(double *x, cov_model *cov, double *v);
+void D3dewijsian(double *x, cov_model *cov, double *v);
+void D4dewijsian(double *x, cov_model *cov, double *v);
 void rangedewijsian(cov_model *cov, range_type* ra);
 int checkdewijsian(cov_model *cov);
 void Inversedewijsian(double *x, cov_model *cov, double *v); 
+void coinitdewijsian(cov_model *cov, localinfotype *li);
 
 /* De Wijsian */
 void DeWijsian(double *x, cov_model *cov, double *v);
@@ -98,7 +143,7 @@ void InverseDeWijsian(double *x, cov_model *cov, double *v);
 
 /* exponential model */
 void exponential(double *x, cov_model *cov, double *v);
-void logexponential(double *x, cov_model *cov, double *v, double *sign);
+void logexponential(double *x, cov_model *cov, double *v, double *Sign);
 void TBM2exponential(double *x, cov_model *cov, double *v);
 void Dexponential(double *x, cov_model *cov, double *v);
 void DDexponential(double *x, cov_model *cov, double *v);
@@ -118,10 +163,21 @@ int init_exp(cov_model *cov, gen_storage *s);
 void do_exp(cov_model *cov, gen_storage *s);
 
 
+
+int checklsfbm(cov_model *cov);
+void rangelsfbm(cov_model VARIABLE_IS_NOT_USED *cov, range_type *range);
+void lsfbm(double *x, cov_model *cov, double *v);
+void Dlsfbm(double *x, cov_model *cov, double *v);
+void DDlsfbm(double *x, cov_model *cov, double *v);
+void D3lsfbm(double *x, cov_model *cov, double *v); 
+void D4lsfbm(double *x, cov_model *cov, double *v); 
+void Inverselsfbm(double *x, cov_model *cov, double *v);
+
+
 // Brownian motion 
 void fractalBrownian(double *x, cov_model *cov, double *v);
-void fractalBrownian(double *x, cov_model *cov, double *v, double *sign);
-void logfractalBrownian(double *x, cov_model *cov, double *v, double *sign);
+void fractalBrownian(double *x, cov_model *cov, double *v, double *Sign);
+void logfractalBrownian(double *x, cov_model *cov, double *v, double *Sign);
 void DfractalBrownian(double *x, cov_model *cov, double *v);
 void DDfractalBrownian(double *x, cov_model *cov, double *v);
 void D3fractalBrownian(double *x, cov_model *cov, double *v);
@@ -144,10 +200,18 @@ void rangeFD(cov_model *cov, range_type* ra);
 void fractGauss(double *x, cov_model *cov, double *v);
 void rangefractGauss(cov_model *cov, range_type* ra);
 
+/*
+void fix(double *x, cov_model *cov, double *v);
+void fix_nonstat(double *x, double *y, cov_model *cov, double *v);
+void covmatrix_fix(cov_model *cov, double *v);
+char iscovmatrix_fix(cov_model *cov);
+int checkfix(cov_model *cov) ;
+*/
+void rangefix(cov_model *cov, range_type* ra);
 
 /* Gausian model */
 void Gauss(double *x, cov_model *cov, double *v);
-void logGauss(double *x, cov_model *cov, double *v, double *sign);
+void logGauss(double *x, cov_model *cov, double *v, double *Sign);
 void DGauss(double *x, cov_model *cov, double *v);
 void DDGauss(double *x, cov_model *cov, double *v);
 void D3Gauss(double *x, cov_model *cov, double *v);
@@ -165,9 +229,20 @@ void do_Gauss(cov_model *cov, gen_storage *s) ;
 //void densGauss(double *x, cov_model *cov, double *v);
 //void simuGauss(cov_model *cov, int dim, double *v);
 
+void bcw(double *x, cov_model *cov, double *v);
+//void logbcw(double *x, cov_model *cov, double *v, double *Sign);
+void Dbcw(double *x, cov_model *cov, double *v);
+void DDbcw(double *x, cov_model *cov, double *v);
+int checkbcw(cov_model *cov);
+void rangebcw(cov_model *cov, range_type* ra);
+void coinitbcw(cov_model *cov, localinfotype *li);
+void ieinitbcw(cov_model *cov, localinfotype *li);
+void Inversebcw(double *x, cov_model *cov, double *v);
+
+
 /* generalised fractal Brownian motion */
 void genBrownian(double *x, cov_model *cov, double *v);
-void loggenBrownian(double *x, cov_model *cov, double *v, double *sign);
+void loggenBrownian(double *x, cov_model *cov, double *v, double *Sign);
 void InversegenBrownian(double *x, cov_model *cov, double *v) ;
 int checkgenBrownian(cov_model *cov);
 void rangegenBrownian(cov_model *cov, range_type* ra);
@@ -175,7 +250,7 @@ void rangegenBrownian(cov_model *cov, range_type* ra);
 
 /* epsC */
 void epsC(double *x, cov_model *cov, double *v);
-void logepsC(double *x, cov_model *cov, double *v, double *sign);
+void logepsC(double *x, cov_model *cov, double *v, double *Sign);
 void DepsC(double *x, cov_model *cov, double *v);
 void DDepsC(double *x, cov_model *cov, double *v);
 int checkepsC(cov_model *cov);
@@ -185,27 +260,27 @@ void inverseepsC(double *x, cov_model *cov, double *v);
 
 /* gencauchy */
 void generalisedCauchy(double *x, cov_model *cov, double *v);
-void loggeneralisedCauchy(double *x, cov_model *cov, double *v, double *sign);
+void loggeneralisedCauchy(double *x, cov_model *cov, double *v, double *Sign);
 void DgeneralisedCauchy(double *x, cov_model *cov, double *v);
 void DDgeneralisedCauchy(double *x, cov_model *cov, double *v);
 int checkgeneralisedCauchy(cov_model *cov);
-bool TypegeneralisedCauchy(Types required, cov_model *cov);
 void rangegeneralisedCauchy(cov_model *cov, range_type* ra);
 void coinitgenCauchy(cov_model *cov, localinfotype *li);
 void ieinitgenCauchy(cov_model *cov, localinfotype *li);
 void InversegeneralisedCauchy(double *x, cov_model *cov, double *v);
 
-void bcw(double *x, cov_model *cov, double *v);
-//void logbcw(double *x, cov_model *cov, double *v, double *sign);
-void Dbcw(double *x, cov_model *cov, double *v);
-void DDbcw(double *x, cov_model *cov, double *v);
-int checkbcw(cov_model *cov);
-bool Typebcw(Types required, cov_model *cov);
-void rangebcw(cov_model *cov, range_type* ra);
-void coinitbcw(cov_model *cov, localinfotype *li);
-void ieinitbcw(cov_model *cov, localinfotype *li);
-void Inversebcw(double *x, cov_model *cov, double *v);
 
+/* bivariate Cauchy bivariate genCauchy bivariate generalized Cauchy */
+
+void biCauchy (double *x, cov_model *cov, double *v);
+void DbiCauchy(double *x, cov_model *cov, double *v);
+void DDbiCauchy(double *x, cov_model *cov, double *v);
+void D3biCauchy(double *x, cov_model *cov, double *v);
+void D4biCauchy(double *x, cov_model *cov, double *v);
+void kappa_biCauchy(int i, cov_model *cov, int *nr, int *nc);
+int checkbiCauchy(cov_model *cov);
+void rangebiCauchy(cov_model VARIABLE_IS_NOT_USED *cov, range_type *range);
+void coinitbiCauchy(cov_model *cov, localinfotype *li);
 
 /* gengneiting */
 void genGneiting(double *x, cov_model *cov, double *v);
@@ -220,13 +295,14 @@ int checkgenGneiting(cov_model *cov);
 void Gneiting(double *x, cov_model *cov, double *v); 
 //void InverseGneiting(double *x, cov_model *cov, double *v);
 void DGneiting(double *x, cov_model *cov, double *v); 
+void DDGneiting(double *x, cov_model *cov, double *v); 
 int checkGneiting(cov_model *cov);
 void rangeGneiting(cov_model *cov, range_type *range);
 
 
 /* hyperbolic */
 void hyperbolic(double *x, cov_model *cov, double *v); 
-void loghyperbolic(double *x, cov_model *cov, double *v, double *sign); 
+void loghyperbolic(double *x, cov_model *cov, double *v, double *Sign); 
 void Dhyperbolic(double *x, cov_model *cov, double *v); 
 int checkhyperbolic(cov_model *cov);
 void rangehyperbolic(cov_model *cov, range_type* ra);
@@ -259,13 +335,15 @@ int checklgd1(cov_model *cov);
 /* Whittle-Matern or Whittle or Besset ---- rescaled form of Whittle-Matern,
     see also there */ 
 void Matern(double *x, cov_model *cov, double *v);
-void logMatern(double *x, cov_model *cov, double *v, double *sign);
+void NonStMatern(double *x, double *y, cov_model *cov, double *v);
+void logNonStMatern(double *x, double*y, 
+		    cov_model *cov, double *v, double *Sign);
+void logMatern(double *x, cov_model *cov, double *v, double *Sign);
 void DMatern(double *x, cov_model *cov, double *v);
 void DDMatern(double *x, cov_model *cov, double *v);
 void D3Matern(double *x, cov_model *cov, double *v);
 void D4Matern(double *x, cov_model *cov, double *v);
 int checkMatern(cov_model *cov);
-bool TypeWM(Types required, cov_model *cov);
 void rangeWM(cov_model *cov, range_type* ra);
 void ieinitWM(cov_model *cov, localinfotype *li);
 void coinitWM(cov_model *cov, localinfotype *li);
@@ -296,7 +374,6 @@ void Inversepower(double *x, cov_model *cov, double *v);
 void TBM2power(double *x, cov_model *cov, double *v);
 void Dpower(double *x, cov_model *cov, double *v);
 int checkpower(cov_model *cov);
-bool Typepower(Types required, cov_model *cov);
 void rangepower(cov_model *cov, range_type* ra);
 
 
@@ -324,11 +401,10 @@ void dospherical(cov_model *cov, gen_storage *s);
 
 /* stable model */
 void stable(double *x, cov_model *cov, double *v);
-void logstable(double *x, cov_model *cov, double *v, double *sign);
+void logstable(double *x, cov_model *cov, double *v, double *Sign);
 void Dstable(double *x, cov_model *cov, double *v);
 void DDstable(double *x, cov_model *cov, double *v);
 int checkstable(cov_model *cov);  
-bool Typestable(Types required, cov_model *cov);
 void rangestable(cov_model *cov, range_type* ra);
 void coinitstable(cov_model *cov, localinfotype *li);
 void ieinitstable(cov_model *cov, localinfotype *li);
@@ -340,6 +416,19 @@ void nonstatLogInversestable(double *x, cov_model *cov, double *L, double *R);
 void stableX(double *x, cov_model *cov, double *v);
 void DstableX(double *x, cov_model *cov, double *v);
 
+
+// bivariate exponential or bivariate stable model
+void kappa_biStable(int i, cov_model *cov, int *nr, int *nc);
+void biStable (double *x, cov_model *cov, double *v);
+void DbiStable (double *x, cov_model *cov, double *v);
+void DDbiStable (double *x, cov_model *cov, double *v);
+void D3biStable (double *x, cov_model *cov, double *v);
+void D4biStable (double *x, cov_model *cov, double *v);
+int checkbiStable(cov_model *cov);
+void rangebiStable(cov_model *cov, range_type *range);
+int initbiStable(cov_model *cov, gen_storage *stor);
+void coinitbiStable(cov_model *cov, localinfotype *li);
+
 /* Stein */
 // see Hypermodel.cc
 
@@ -358,11 +447,12 @@ void Inversewave(double *x, cov_model *cov, double *v);
 int initwave(cov_model *cov, gen_storage *s);
 void spectralwave(cov_model *cov, gen_storage *s, double *e); 
 
-extern double Besselupperbound[Nothing + 1];
-
 /* Whittle-Matern or Whittle or Besset */ 
 void Whittle(double *x, cov_model *cov, double *v);
-void logWhittle(double *x, cov_model *cov, double *v, double *sign);
+void NonStWhittle(double *x, double *y, cov_model *cov, double *v);
+void logNonStWhittle(double *x, double*y, 
+		    cov_model *cov, double *v, double *Sign);
+void logWhittle(double *x, cov_model *cov, double *v, double *Sign);
 void TBM2Whittle(double *x, cov_model *cov, double *v);
 void DWhittle(double *x, cov_model *cov, double *v);
 void DDWhittle(double *x, cov_model *cov, double *v);
@@ -377,7 +467,7 @@ double LogMixDensW(double *x, double logV, cov_model *cov);
 void InverseWhittle(double *x, cov_model *cov, double *v);
 
 double WM(double x, double nu, double factor);
-double logWM(double x, double nu, double factor);
+double logWM(double x, double nu1, double nu2, double factor);
 double DWM(double x, double nu, double factor);
 
 void kappa_biGneiting(int i, cov_model *cov, int *nr, int *nc);
@@ -385,7 +475,6 @@ void biGneiting(double *x, cov_model *cov, double *v);
 void DbiGneiting(double *x, cov_model *cov, double *v);
 void DDbiGneiting(double *x, cov_model *cov, double *v);
 int checkbiGneiting(cov_model *cov);
-sortsofparam paramtype_biGneiting(int k, int row, int col) ;
 void rangebiGneiting(cov_model *cov, range_type* ra);
 int initbiGneiting(cov_model *cov, gen_storage *s);
 
@@ -398,7 +487,7 @@ void DUser(double *x, cov_model *cov, double *v);
 void DDUser(double *x, cov_model *cov, double *v);
 int checkUser(cov_model *cov);
 void rangeUser(cov_model *cov, range_type *ra);
-bool TypeUser(Types required, cov_model *cov);
+bool TypeUser(Types required, cov_model *cov, int depth);
 
 /*
 void userMatrix(double *x, cov_model *cov, double *v);
@@ -415,13 +504,15 @@ void kappa_userMatrix(int i, cov_model *cov, int *nr, int *nc);
 void kappa_biWM(int i, cov_model *cov, int *nr, int *nc);
 void biWM2(double *x, cov_model *cov, double *v);
 void biWM2D(double *x, cov_model *cov, double *v);
+void biWM2DD(double *x, cov_model *cov, double *v);
+void biWM2D3(double *x, cov_model *cov, double *v);
+void biWM2D4(double *x, cov_model *cov, double *v);
 int checkbiWM2(cov_model *cov);
-sortsofparam paramtype_biWM(int k, int row, int col);
 void rangebiWM2(cov_model *cov, range_type* ra);
 int initbiWM2(cov_model *cov, gen_storage *s);
+void coinitbiWM2(cov_model *cov, localinfotype *li);
 
 
-sortsofparam paramtype_biWM(int k, int row, int col);
 void rangebiWM2(cov_model *cov, range_type* ra);
 
 
@@ -429,9 +520,41 @@ void kappa_parsWM(int i, cov_model *cov, int *nr, int *nc);
 void parsWM(double *x, cov_model *cov, double *v);
 void parsWMD(double *x, cov_model *cov, double *v);
 int checkparsWM(cov_model *cov);
-sortsofparam paramtype_parsWM(int k, int row, int col);
 void rangeparsWM(cov_model *cov, range_type* ra);
 
  
+void Mathminus(double *x, cov_model *cov, double *v);
+void Mathplus(double *x, cov_model *cov, double *v);
+void Mathmult(double *x, cov_model *cov, double *v);
+void Mathdiv(double *x, cov_model *cov, double *v);
+void Mathc(double *x, cov_model *cov, double *v);
+void rangec(cov_model VARIABLE_IS_NOT_USED *cov, range_type *range);
+int check_c(cov_model *cov);
+void Mathis(double *x, cov_model *cov, double *v);
+void rangeMathIs(cov_model *cov, range_type *range);
+void Mathbind(double *x, cov_model *cov, double *v);
+int check_bind(cov_model *cov);
+void proj(double *x, cov_model *cov, double *v);
+int checkproj(cov_model *cov);
+void rangeproj(cov_model VARIABLE_IS_NOT_USED *cov, range_type *range);
+
+
+void kappa_math(int i, cov_model *cov, int *nr, int *nc);
+int checkMath(cov_model *cov);
+void rangeMath(cov_model *cov, range_type *range);
+
+#define MATH_DEFAULT_0(VARIAB)						\
+  int i,								\
+    variables = VARIAB;							\
+  double w[MAXPARAM];							\
+    for (i=0; i<variables; i++) {					\
+      cov_model *sub = cov->kappasub[i];				\
+      if (sub != NULL) {						\
+	COV(x, sub, w + i);						\
+      } else {  w[i] = P0(i);	}					\
+    }									\
+  // printf("sin %f w=%f;%f %f %d %d nr = %d %d\n", x[0], w[0], w[1], P0(1), variables, p[1], cov->nrow[0] > 0, cov->nrow[1] > 0);
+
+#define MATH_DEFAULT MATH_DEFAULT_0(CovList[cov->nr].kappas)
 
 #endif /* Primitives_H*/
