@@ -4,7 +4,7 @@
 
  (library for simulation of random fields)
 
- Copyright (C) 2001 -- 2014 Martin Schlather, 
+ Copyright (C) 2001 -- 2017 Martin Schlather, 
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,42 +23,40 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include <R.h>
 #include <Rdefines.h>
-#include <math.h>  
+#include <Rmath.h>  
 #include <stdio.h>  
-#include <stdlib.h>
- 
+//#include <stdlib.h>
 #include <string.h>
+#include <R_ext/Linpack.h>
 #include "RF.h"
 #include "primitive.h"
-#include <R_ext/Linpack.h>
-#include "Covariance.h"
-
-
-void addkappa(int i, const char *n, SEXPTYPE t, Types ParamType) {
-  cov_fct *C = CovList + currentNrCov - 1;
-
-  // printf("%s\n", C->name);
-
-  assert(n[0] != '\0' && 
-	 (n[0] != 'k' || n[1] != '\0') // reserved for standard parameter names
-	 && (n[0] != 'C' || (n[1] != '\0' && (n[1] < '1' ||  n[1] > '9')))
-                                        // reserved for standard submodel names
-	 );
-  assert(i < C->kappas);
-  strcopyN(C->kappanames[i], n, PARAMMAXCHAR);
-  C->kappatype[i] = t;
-  C->kappaParamType[i] = ParamType;
-  assert(strcmp(n, FREEVARIABLE) || C->internal);
-  if (t >= LISTOF) assert(strcmp(C->kappanames[0], "element") == 0 
-			  ||  strcmp(C->name, "select") == 0
-			  );
-}
+#include "Operator.h"
+#include "variogramAndCo.h"
 
 
 void addkappa(int i, const char *n, SEXPTYPE t) {
-  addkappa(i, n, t, RandomType);
+  cov_fct *C = CovList + currentNrCov - 1;
+
+  //   if (ParamType != 7)  printf("%s %s %d\n", C->name, n, ParamType);
+
+  assert(n[0] != '\0' && 
+	 (n[0] != ONEARGUMENT_NAME || n[1] != '\0') // reserved for standard parameter names
+	 && (n[0] != DEFAULT_SUBNAME || 
+	     (n[1] != '\0' && (n[1] < '1' ||  n[1] > '9')))
+	 // reserved for standard submodel names
+	 );
+  assert(i < C->kappas);
+  // assert(STRCMP(n, ELEMENT) || C->check == checkfix || C->check == checkmixed);
+  strcopyN(C->kappanames[i], n, PARAMMAXCHAR);
+  C->kappatype[i] = t;
+  assert(STRCMP(n, FREEVARIABLE) || C->internal);
+  // if (t >= LISTOF) assert(STRCMP(C->kappanames[0], ELEMENT) == 0 
+  //			  ||  C->check == checkselect
+  //			  );
 }
 
+
+
 void kappanames(const char* n1, SEXPTYPE t1) {
   assert({cov_fct *C = CovList + currentNrCov - 1; C->kappas == 1;});
   addkappa(0, n1, t1);
@@ -262,55 +260,55 @@ void kappanames(const char* n1, SEXPTYPE t1, const char* n2, SEXPTYPE t2,
   addkappa(15, n16, t16);
 }
 
-sortsofparam standard_paramtype(int k, int VARIABLE_IS_NOT_USED row, int VARIABLE_IS_NOT_USED col) {
+sortsofparam SortOf(cov_model *cov,int k, int row, int col) {
+  cov_fct *C = CovList + cov->nr;
+  if (C->sortof != NULL) return C->sortof(k, row, col);
   // for MLE
 // k non-negative: k-th parameter
 // k negative: (i-1)th submodel
-  return k<0 ? VARPARAM : CRITICALPARAM;
-// k<0: varparam used to indicate that variance is allowed for submodel,
-// see recursive call getnaposition; E.g. not allowed for second submodel
-// of nsst: the variance parameter is something between scale and variance
-}
+  if (k >= C->kappas) BUG;
 
-sortsofparam ignoreall_paramtype(int VARIABLE_IS_NOT_USED k, int VARIABLE_IS_NOT_USED row, int VARIABLE_IS_NOT_USED col) {
-  // for MLE
-// k non-negative: k-th parameter
-// k negative: (i-1)th submodel
-  return IGNOREPARAM;
-// k<0: varparam used to indicate that variance is allowed for submodel,
-// see recursive call getnaposition; E.g. not allowed for second submodel
-// of nsst: the variance parameter is something between scale and variance
-}
-
-sortsofparam uncritical_paramtype(int k, int VARIABLE_IS_NOT_USED row, int VARIABLE_IS_NOT_USED col) {
-  // for MLE
-// k non-negative: k-th parameter
-// k negative: (i-1)th submodel
-  return k<0 ? VARPARAM : ANYPARAM;
+  return k<0 ? VARPARAM : C->sortof_tab[k];
 // k<0: varparam used to indicate that variance is allowed for submodel,
 // see recursive call getnaposition; E.g. not allowed for second submodel
 // of nsst: the variance parameter is something between scale and variance
 }
 
 
-
-void add_paramtype(paramtype_fct paramtype) {
+void change_sortof(int i, sortsofparam sort) {
   cov_fct *C = CovList + currentNrCov - 1;
-  C->paramtype = paramtype;
+  assert(i >= 0 && i < C->kappas);
+  C->sortof_tab[i] = sort;
+}
+
+void change_typeof(int i, Types type) {
+  cov_fct *C = CovList + currentNrCov - 1;
+  assert(i >= 0 && i < C->kappas);
+  assert(type == RandomType ||  // parameter might be random
+	 type == ShapeType ||   // parameter must be determinist
+	 type == RandomOrShapeType || 
+	 (type >= NN1 && type <= NN4));
+  C->kappaParamType[i] = type;
+}
+
+
+void add_sortof(sortof_fct sortof) {
+  cov_fct *C = CovList + currentNrCov - 1;
+  C->sortof = sortof;
 }
 
 void addsub(int i, const char *n) {
   cov_fct *C = CovList + currentNrCov - 1;
   int j;
-  assert(n[0] != 'k' && n[0] != 'C');
+  assert(n[0] != ONEARGUMENT_NAME && n[0] != DEFAULT_SUBNAME);
   assert(i < MAXSUB);
 
   strcopyN(C->subnames[i], n, PARAMMAXCHAR);
   C->subintern[i] = false;
   for (j=0; j<C->kappas; j++) {
     if (C->kappanames[j] == NULL)
-      error("give all names for the parameters first");
-    if ((C->subintern[i] = strcmp(C->kappanames[j], C->subnames[i]) == 0))
+      ERR("give all names for the parameters first");
+    if ((C->subintern[i] = STRCMP(C->kappanames[j], C->subnames[i]) == 0))
       break;
   }
 }
@@ -352,12 +350,19 @@ void subnames(const char* n1, const char* n2, const char* n3, const char* n4,
 }
 
 
-int xxx(int x) {return (int) pow(10, (double) x);}
-void crash(cov_model *cov) {
-    PRINTF("crash!!!!\n");
-    char m[1]; memcpy(m, cov, xxx(3));  PRINTF("%s\n", m); // not MEMCOPY
+int xxx(int x) {return (int) POW(10, (double) x);}
+
+
+void crash() {  
+#ifdef SCHLATHERS_MACHINE
+  //  int i;PRINTF("%d\n", i);char m[1];m[i] = m[i-9] + 4; if (m[0]) i++; else i--; PRINTF("%s\n", m); // not MEMCOPY
+  //  int *x = (int*) MALLOC(1000000);  f ree(x);  f ree(x); x[100] = 100;
+#else 
+ BUG;
+#endif
 }
-void crash() {cov_model *cov=NULL; crash(cov);}
+
+
 
 void ErrCovX(double VARIABLE_IS_NOT_USED *x, cov_model *cov,
 	     double VARIABLE_IS_NOT_USED *v, const char *name) {
@@ -365,23 +370,23 @@ void ErrCovX(double VARIABLE_IS_NOT_USED *x, cov_model *cov,
   PRINTF("\nErr%s %s [%d] gatter=%d:\n", name,
 	 NICK(cov), cov->nr, cov->gatternr); // ok
   if (PL >= PL_ERRORS){
-    PMI(cov, "ErrCov");//
-    crash(cov);
+    PMI(cov);//
+    crash();
   }
   ERR("unallowed or undefined call of function");
 }
-void ErrCov(double *x, cov_model *cov, double *v) { ErrCovX(x, cov, v, "cov");}
+void ErrCov(double *x, cov_model *cov, double *v) { ErrCovX(x, cov, v, "Cov");}
 void ErrD(double *x, cov_model *cov, double *v) { ErrCovX(x, cov, v, "D");}
 void ErrRnd(double *x, cov_model *cov, double *v) { ErrCovX(x, cov, v, "Rnd");}
 
 void ErrLogCov(double VARIABLE_IS_NOT_USED *x, cov_model *cov, 
 	       double VARIABLE_IS_NOT_USED *v, 
-	       double VARIABLE_IS_NOT_USED *sign) {
+	       double VARIABLE_IS_NOT_USED *Sign) {
   // 
   PRINTF("\nErrlogCov %s:\n", NICK(cov));
   if (PL >=  PL_ERRORS) {
-    PMI(cov, "ErrlogCov");//
-    crash(cov); 
+    PMI(cov);//
+    crash(); 
   }
   ERR("unallowed or undefined call of function (log)");
 }
@@ -391,29 +396,29 @@ void ErrCovNonstat(double VARIABLE_IS_NOT_USED *x,
   PRINTF("\nErrCovNonstat %s: (%d)\n", NICK(cov), cov->nr);
   //  APMI(cov->calling);
   if (PL >= PL_ERRORS) {
-    PMI(cov->calling, "ErrCovNonstat"); //
-    crash(cov);
+    PMI(cov->calling); //
+    crash();
   }
-  ERR("unallowed or undefined call of non-domain function");
+  ERR1("unallowed or undefined call of '%s' as a kernel", NAME(cov));
 }
 void ErrLogCovNonstat(double VARIABLE_IS_NOT_USED *x, 
 		      double VARIABLE_IS_NOT_USED *y, cov_model *cov,
 		      double VARIABLE_IS_NOT_USED *v, 
-		      double VARIABLE_IS_NOT_USED *sign) {
+		      double VARIABLE_IS_NOT_USED *Sign) {
   PRINTF("\nErrlogCovNonstat %s: (%d)\n", NICK(cov), cov->nr);
   if (PL >= PL_ERRORS) {
-    PMI(cov->calling, "ErrLogCovNonstat"); //
-    crash(cov);
+    PMI(cov->calling);//
+    crash();
   }
-  ERR("unallowed or undefined call of non-domain function (log)");
+  ERR1("unallowed or undefined call of '%s' (log) as a kernel", NAME(cov));
 }
 void Errspectral(cov_model *cov,
 		 gen_storage VARIABLE_IS_NOT_USED *s, 
 		 double VARIABLE_IS_NOT_USED *e) {
   PRINTF("\nErrlogCovNonstat %s: (%d)\n", NICK(cov), cov->nr);
  if (PL >= PL_ERRORS) {
-   PMI(cov->calling, "Errspectral"); //
-    crash(cov);
+   PMI(cov->calling);//
+    crash();
   }
   ERR("unallowed or undefined call of spectral function");
 } 
@@ -426,8 +431,8 @@ void ErrInverseNonstat(double VARIABLE_IS_NOT_USED *v, cov_model *cov,
   PRINTF("\nErrCovNonstat %s: (%d)\n", NICK(cov), cov->nr);
   //  APMI(cov->calling);
   if (PL >= PL_ERRORS) {
-    PMI(cov->calling, "ErrCovNonstat"); //
-    crash(cov);
+    PMI(cov->calling); //
+    crash();
   }
   ERR("unallowed or undefined call of non-domain function (inverse)");
 }
@@ -448,6 +453,7 @@ void kappasize1(int VARIABLE_IS_NOT_USED i,
 
 void rangeOK(cov_model VARIABLE_IS_NOT_USED *cov, 
 	     range_type VARIABLE_IS_NOT_USED *range) { 
+  //  printf("rangeOK %s\n", NAME(cov));
   assert(CovList[cov->nr].kappas == 0);
 }
 
@@ -462,13 +468,13 @@ int struct_statiso(cov_model *cov, cov_model **newmodel) {
   
   if (hasAnyShapeRole(cov)) {
     int i,
-      vdim = cov->vdim2[0];
+      vdim = cov->vdim[0];
     for (i=0; i<vdim; i++) cov->mpp.maxheights[i] = 1.0;
   }
 
   switch (cov->role) {
   case ROLE_POISSON_GAUSS :
-    SERR1("Unexpected call of'struct' by '%s'", NICK(cov));
+    SERR1("Unexpected call of 'struct' by '%s'", NICK(cov));
     break;      
   case ROLE_POISSON :
     if (C->finiterange == true) {
@@ -485,8 +491,12 @@ int struct_statiso(cov_model *cov, cov_model **newmodel) {
 int struct_failed(cov_model *cov, cov_model VARIABLE_IS_NOT_USED **newmodel) {
   //  PMI(cov);
   //  printf("%ld %ld %ld\n", CovList[cov->nr].Struct, CovList[DISTRIBUTION].Struct, struct_failed);  crash();
-  SERR3("initialization failed --  model '%s' (%d) does not fit (yet) the properties required by '%s'.",
-	NICK(cov), cov->nr, cov->calling == NULL ? "<null>" : NICK(cov->calling)); 
+  SERR4("initialization failed --  model '%s' (%d) does not fit (yet) the properties required by '%s'. %s",
+	NICK(cov), cov->nr, 
+	cov->calling == NULL ? "<null>" : NICK(cov->calling),
+	cov->secondarygatternr == MISMATCH ? "" : 
+	"NOTE THAT THE ERROR CAN ALSO BE CAUSED BY COORDINATE TRANSFORMATION\n"
+	); 
 }
 
 int initOK(cov_model *cov, gen_storage *s) {
@@ -497,7 +507,7 @@ int initOK(cov_model *cov, gen_storage *s) {
   for (i=0; i<nk; i++) {
     cov_model *ks = cov->kappasub[i];
     if (ks != NULL) {
-      if (isRandom(C->kappaParamType[i])) {
+      if (isRandom((Types) C->kappaParamType[i])) {
 	random = true;
 	if ((err = INIT(ks, cov->mpp.moments, s)) != NOERROR) return err;
       } else {
@@ -546,43 +556,6 @@ int init_statiso(cov_model *cov, gen_storage *s) {
 }
 
 
-void standard_likelihood(double VARIABLE_IS_NOT_USED *x, cov_model *cov, 
-			 double *v){ 
-  int i,
-    nsub = cov->nsub,
-    kappas = CovList[cov->nr].kappas;
-  cov_model *sub;
-  double dummy;
-  *v = 0.0;
-  for (i=0; i<kappas; i++) {
-    if ( (sub = cov->kappasub[i]) != NULL) {
-      if (TypeConsistency(ProcessType, sub) && 
-	  !TypeConsistency(TrendType, sub))
-	error("Baysian modelling currently only works with simple models");
-      if (TypeConsistency(RandomType, sub)) {
-	if (CovList[sub->nr].kappatype[i]!=REALSXP)
-	  error("currently only real-valued parameters can be random");
-	VTLG_DLOG(P(i), sub, &dummy);
-	*v += dummy;
-      } else { 
-	VTLG_DLOG(NULL, sub, &dummy);
-	*v += dummy;
-      }
-    }
-  }
-  
-  for (i=0; i<nsub; i++) {
-    sub = cov->sub[i];
-    if (sub->deterministic) continue;
-    if (TypeConsistency(RandomType, sub)) {
-      error("Baysian modelling only works with simple models.");
-    }    
-    VTLG_DLOG(NULL, sub, &dummy);
-    *v += dummy;
-  }
-}
-
-
 void doOK(cov_model VARIABLE_IS_NOT_USED *cov, gen_storage VARIABLE_IS_NOT_USED *s){
 }
 
@@ -612,36 +585,37 @@ void do_statiso(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *s) {
 }
 
 static int badname = -1;
-void nickname(const char *name, int nr) {
+void nickname(const char *name, int nr, int type) {
   char dummy[MAXCHAR];
   cov_fct *C = CovList + nr; // nicht gatternr 
   
-  int sl = strlen(CAT_TYPENAMES[C->Type]);  
+  int sl = STRLEN(CAT_TYPENAMES[type]);  
   strcopyN(dummy, name, MAXCHAR-sl);
-  sprintf(C->nick, "%s%s", CAT_TYPENAMES[C->Type], dummy);
-  strcpy(CovNickNames[nr], C->nick);
+  //printf("%s %s\n",  CAT_TYPENAMES[type], dummy);
+  SPRINTF(C->nick, "%s%s", CAT_TYPENAMES[type], dummy);
+  STRCPY(CovNickNames[nr], C->nick);
 
-  if ((int) strlen(name) >= (int) MAXCHAR - sl) {
+  if ((int) STRLEN(name) >= (int) MAXCHAR - sl) {
     badname = nr;
   } else {
     if (badname >= 0 && badname != nr) 
-      PRINTF("Warning! Nick name is truncated to `%s'.\n", 
+      PRINTF("Warning! Nick name is truncated to '%s'.\n", 
 	     CovList[badname].nick);
     badname = -1;
   }
 }
 
-void insert_name(int curNrCov, const char *name) {
+void insert_name(int curNrCov, const char *name, int type) {
   cov_fct *C = CovList + curNrCov;
   char dummy[MAXCHAR];
   strcopyN(dummy, name, MAXCHAR);
-  strcpy(CovNames[curNrCov], dummy);
-  strcpy(C->name, dummy);
-  if (strlen(name)>=MAXCHAR) {
-    PRINTF("Warning! Covariance name is truncated to `%s'.\n", C->name);
+  STRCPY(CovNames[curNrCov], dummy);
+  STRCPY(C->name, dummy);
+  if (STRLEN(name)>=MAXCHAR) {
+    PRINTF("Warning! Covariance name is truncated to '%s'.\n", C->name);
   }
-  assert(strcmp(InternalName, name));
-  nickname(name, curNrCov);
+  assert(STRCMP(InternalName, name));
+  nickname(name, curNrCov, type);
 }
 
 void StandardCovariance(cov_model *cov, double *v){
@@ -650,8 +624,8 @@ void StandardCovariance(cov_model *cov, double *v){
 void StandardCovMatrix(cov_model *cov, double *v) { 
   CovarianceMatrix(cov, v); 
 }
-void StandardInverseCovMatrix(cov_model *cov, double *v) { 
-  InverseCovMatrix(cov, v); 
+void StandardInverseCovMatrix(cov_model *cov, double *inverse, double *det) { 
+  InverseCovMatrix(cov, inverse, det); 
 }
 void StandardVariogram(cov_model *cov, double *v) {
   //assert(false);
@@ -660,9 +634,6 @@ void StandardVariogram(cov_model *cov, double *v) {
 void StandardPseudoVariogram(cov_model *cov, double *v) {
   CovVario(cov, false, true, v); 
 }
-void StandardSelectedCovMatrix(cov_model *cov, int *sel, int nsel, double *v) { 
-  SelectedCovMatrix(cov, sel, nsel, v); 
-}
 
 void StandardInverseNonstat(double *v, cov_model *cov,
 			    double *left, double *right) {
@@ -685,7 +656,7 @@ void StandardLogInverseNonstat(double *v, cov_model *cov,
 			    double *left, double *right) {
   assert(CovList[cov->nr].inverse != NULL);
   double x, 
-    w = exp(*v);
+    w = EXP(*v);
   int d,
     dim = cov->tsdim;
   
@@ -700,11 +671,11 @@ void StandardLogInverseNonstat(double *v, cov_model *cov,
 
 void StandardNonLogDistrD(double *x, cov_model *cov, double *D) {
   VTLG_DLOG(x, cov, D);
-  *D = exp(*D);
+  *D = EXP(*D);
 }
 
 
-char isTrue(cov_model VARIABLE_IS_NOT_USED *cov) {error("isTrue may not be used anymore"); return true;}
+char isTrue(cov_model VARIABLE_IS_NOT_USED *cov) {ERR("isTrue may not be used anymore"); return true;}
 char isFalse(cov_model VARIABLE_IS_NOT_USED *cov) {return false;}
 
 void InverseFiniteRange(double VARIABLE_IS_NOT_USED *x, cov_model VARIABLE_IS_NOT_USED *cov, double *v){ *v = 1.0; }
@@ -717,10 +688,10 @@ void InverseIsotropic(double *U, cov_model *cov, double *inverse){
 #define tol 1e-8
 #define max_it 30
   
-  if (cov->vdim2[0] != cov->vdim2[1]) BUG;
-  int vdim = cov->vdim2[0],
+  if (cov->vdim[0] != cov->vdim[1]) BUG;
+  int vdim = cov->vdim[0],
     vdimSq = vdim * vdim;
-  if (cov->Sinv == NULL) NEW_STORAGE(Sinv, INV, inv_storage);
+  if (cov->Sinv == NULL) NEW_STORAGE(inv);
   ALLOC_NEW(Sinv, v, vdimSq, v);
   ALLOC_NEW(Sinv, wert, vdimSq, wert);
  
@@ -742,7 +713,7 @@ void InverseIsotropic(double *U, cov_model *cov, double *inverse){
     x *= step;
   }
   if (i >= max_it) {
-    *inverse = fabs(*v - u) <= fabs(*wert - u) ? 0.0 : RF_INF;
+    *inverse = FABS(*v - u) <= FABS(*wert - u) ? 0.0 : RF_INF;
     return;
   }
   *inverse = *wert;
@@ -774,7 +745,6 @@ void createmodel(const char *name, Types type, int kappas, size_fct kappasize,
 		 domain_type domain, isotropy_type isotropy,
 		 checkfct check, rangefct range, int vdim, pref_shorttype pref,
 		 int maxdim, ext_bool finiterange, int monotone) {
-  int i;
   cov_fct *C = CovList + currentNrCov;
   bool 
     stat_iso = domain == XONLY && isotropy == ISOTROPIC;
@@ -786,19 +756,33 @@ void createmodel(const char *name, Types type, int kappas, size_fct kappasize,
   assert(currentNrCov>=0);
   if (currentNrCov>=MAXNRCOVFCTS) {
     char msg[150];
-    sprintf(msg, "maximum number of covariance models reached. Last included  model is '%s'.", CovList[MAXNRCOVFCTS-1].name);
-    warning(msg);
+    SPRINTF(msg, "maximum number of covariance models reached. Last included  model is '%s'.", CovList[MAXNRCOVFCTS-1].name);
+    ERR(msg);
   }
-  
+
   if (PL >= PL_DETAILS)
     PRINTF("%d %s vdim=%d statiso=%d iso=%d\n", currentNrCov, name, vdim, stat_iso, isotropy); 
-  C->Type = type; // ganz vorne 
+  C->TypeFct = NULL;
+  assert(type >=0 && type <= OtherType);
+
+  assert((isotropy >= 0 && (isotropy != ISO_MISMATCH || type == RandomType)) 
+	 || type == MathDefinition);
+
+
+  C->Typi[0] = type; // ganz vorne 
   //  printf("%d %s vdim=%d statiso=%d iso=%d\n", currentNrCov, name, vdim, stat_iso, isotropy); 
   // printf("type = %d\n", type);
-  assert(type >=0 && type <= OtherType);
-  C->TypeFct = NULL;
+  C->Isotropy[0] = isotropy;
+  C->variants = 1;
+  if ((finiterange == true && isPosDef(type) && vdim == SCALAR) || 
+      monotone == COMPLETELY_MON) {
+    C->Isotropy[C->variants] = SPHERICAL_ISOTROPIC;
+    C->Typi[C->variants] = PosDefType;
+    C->variants++;
+    //printf("name %s\n", name);
+   }
+  insert_name(currentNrCov, name, type);
 
-  insert_name(currentNrCov, name);
 
   //if (domain == PARAM_DEP) printf("%s\n", C->nick);
 
@@ -810,22 +794,31 @@ void createmodel(const char *name, Types type, int kappas, size_fct kappasize,
   assert(kappas >= 0 && kappas <= MAXPARAM);
   C->minsub = C->maxsub = 0;
   C->domain = domain; 
-  C->isotropy = isotropy;
   C->vdim = vdim; assert(vdim != MISMATCH);
-
+  
   //  if (vdim == SUBMODEL_DEP) printf("SUBMODELDEP %s\n", name);
   //  if (vdim == PREVMODEL_DEP) printf("prevMODELDEP %s\n", name);
 
   C->maxdim = maxdim;
   C->maxmoments = 0;
   assert(maxdim != MISMATCH);
-  for (i=0; i<kappas; i++) {
-    sprintf(C->kappanames[i], "k%d", i); // default (repeated twice)
+  for (int i=0; i<kappas; i++) {
+    SPRINTF(C->kappanames[i], "%c%d", ONEARGUMENT_NAME, i); // default (repeated twice)
     C->kappatype[i] = REALSXP;
   }
   C->kappasize = (kappasize == NULL) ? kappasize1 : kappasize;
-  C->paramtype=standard_paramtype;
-    
+  C->sortof = NULL;
+  if (isProcess(type)) {
+    // UNBEDEINGT BIS ZUM SCHLUSS LAUFEN LASSEN, DA VERDECKTE PARAMTER
+    for (int i=0; i<MAXPARAM; i++) C->sortof_tab[i] = FORBIDDENPARAM;
+  } else {
+    for (int i=0; i<MAXPARAM; i++) C->sortof_tab[i] = ANYPARAM;
+  }
+  if (type == MathDefinition)
+    for (int i=0; i<MAXPARAM; i++) C->kappaParamType[i] = ShapeType;
+  else 
+    for (int i=0; i<MAXPARAM; i++) C->kappaParamType[i] = RandomType;
+ 
   if (kappas==0) {
     assert(range == NULL);
     C->range=rangeOK; 
@@ -835,8 +828,7 @@ void createmodel(const char *name, Types type, int kappas, size_fct kappasize,
   }
   C->check = check;
   assert(check != NULL);
-  for (i=0; i<Forbidden; i++)
-    C->implemented[i] = NOT_IMPLEMENTED;
+  for (int i=0; i<Forbidden; C->implemented[i++] = NOT_IMPLEMENTED);
   C->internal = false;
   C->Specific = MISMATCH;
   assert(finiterange != MISMATCH);
@@ -848,8 +840,12 @@ void createmodel(const char *name, Types type, int kappas, size_fct kappasize,
 				   || type== GaussMethodType
 				   || type== BrMethodType 
 				   || type==ProcessType)));
-  assert(monotone != PARAM_DEP);
+  // assert(monotone != PARAM_DEP);
   C->Monotone = monotone;
+  C->ptwise_definite = !isShape(type) && type != MathDefinition ? pt_mismatch
+    : isTcf(type) || isBernstein(monotone)
+    || (isVariogram(type) && isMonotone(monotone) && C->vdim == 1)
+    ? pt_posdef : pt_unknown;
 
 
   MEMCOPY(C->pref, pref, sizeof(pref_shorttype));
@@ -875,7 +871,8 @@ void createmodel(const char *name, Types type, int kappas, size_fct kappasize,
   C->drawmix = NULL;
   C->logmixdens = NULL;
 
-  if (isNegDef(type) || isShape(type)) C->logD = standard_likelihood;
+  //  if (isVariogram(type) || isShape(type) || 
+  //    type == MathDefinition) C->logD = standard_likelihood;
 
   C->Struct = stat_iso ? struct_statiso : struct_failed;
   C->Init = stat_iso ? init_statiso : init_failed; // !!!! see isDummyInit below
@@ -891,23 +888,28 @@ void createmodel(const char *name, Types type, int kappas, size_fct kappasize,
   C->inversecovmatrix = StandardInverseCovMatrix;
   C->variogram = StandardVariogram;
   C->pseudovariogram = StandardPseudoVariogram;
-  C->selectedcovmatrix = StandardSelectedCovMatrix;
   C->is_covariance=C->is_covmatrix=C->is_inversecovmatrix=C->is_variogram
     = C->is_pseudovariogram = isFalse;
 
   C->TaylorN = C->TailN = MISMATCH;
  
+  //  printf("%s %s (%s)\n", C->name, TYPENAMES[C->Typi[0]], TYPENAMES[type]);
+
   currentNrCov++;
+
 }
 
+
 bool isDummyInit(initfct Init) {
   return Init == init_statiso || Init == init_failed;
 }
 
 
 int CopyModel(const char *name, int which) {
-  memcpy(CovList + currentNrCov, CovList + which, sizeof(cov_fct));  
-  insert_name(currentNrCov, name);
+  MEMCOPYX(CovList + currentNrCov, CovList + which, sizeof(cov_fct)); 
+  int type = CovList[which].Typi[0];
+  assert(type <= OtherType);
+  insert_name(currentNrCov, name, type);
   currentNrCov++;
   return currentNrCov - 1;
 }
@@ -916,7 +918,8 @@ int CopyModel(const char *name, int which) {
 int CopyModel(const char *name, int which, Types type) {
   CopyModel(name, which);
   int nr = currentNrCov - 1;
-  CovList[nr].Type = type;
+  assert(CovList[nr].variants == 1);
+  CovList[nr].Typi[0] = type;
   return nr;
 }
 
@@ -938,7 +941,9 @@ int CopyModel(const char *name, int which, checkfct check) {
 
 void nickname(const char *name) {
   int nr = currentNrCov - 1;
-  nickname(name, nr);
+  int type = CovList[nr].Typi[0];
+  assert(type <= MathDefinition);
+  nickname(name, nr, type);
 }
 
 int IncludePrim(const char *name, Types type,  int kappas, 
@@ -1004,16 +1009,24 @@ void make_internal() {
 
 
 // extern ?!
-int IncludeModel(const char *name, Types type, 
-		 int minsub, int maxsub, int kappas,
+int IncludeModel(const char *name, Types type, int minsub, int maxsub, int kappas,
 		 size_fct kappasize,
 		 domain_type domain, isotropy_type isotropy,
 		 checkfct check, rangefct range, pref_type pref, 
-		 bool internal, int vdim, int maxdim, ext_bool finiterange,
-		 int monotonicity) {  
+		 int internal, int vdim, 
+		 int maxdim, ext_bool finiterange, int monotonicity) {  
   createmodel(name, type, kappas, kappasize, domain, isotropy, check, range,
 	      vdim, pref, maxdim, finiterange, monotonicity);
   //    assert(maxsub > 0); // check deleted 25. nov 2008 due to nugget 
+  // printf("name = %s\n", name);
+  assert(isotropy != PREVMODELI || maxsub != 0 || type == MathDefinition
+	 || STRCMP("trend", name) == 0
+	 || STRCMP("U", name) == 0 
+	 || STRCMP("whittle", name) == 0 
+	 || STRCMP("matern", name) == 0 
+	 || STRCMP("idcoord", name) == 0
+	 || STRCMP("constant", name) == 0);
+
   assert(maxsub >= minsub && maxsub <= MAXSUB);
   assert(check != checkOK || maxsub==0);
   int i, 
@@ -1053,15 +1066,14 @@ int IncludeModel(const char *name, Types type,
     }
   } else {
     for (i=0; i<maxsub; i++) {      
-      sprintf(C->subnames[i], "C%d", i); // default (repeated twice)
+      SPRINTF(C->subnames[i], "C%d", i); // default (repeated twice)
       C->subintern[i] = false;
     }
   }
   return nr;
 }
 
-int IncludeModel(const char *name, Types type, 
-		 int minsub, int maxsub, int kappas, 
+int IncludeModel(const char *name, Types type, int minsub, int maxsub, int kappas, 
 		 domain_type domain, isotropy_type isotropy,
 		 checkfct check, rangefct range, pref_type pref, 
 		 int maxdim, ext_bool finiterange, int monotonicity) {
@@ -1072,13 +1084,53 @@ int IncludeModel(const char *name, Types type,
 		 SCALAR, maxdim, finiterange, monotonicity);
 }
 
+bool addvariantOK(Types type, isotropy_type iso) {
+  int nr = currentNrCov - 1;
+  cov_fct *C = CovList + nr; // nicht gatternr
+  if
+ (C->variants >=  MAXVARIANTS) return false;
+  // printf("A %s\n", C->name);
+  if (C->Isotropy[0] == PREVMODELI || iso == PREVMODELI)
+    if (C->check != checkconstant && C->Typi[0] != MathDefinition &&
+	C->check != checkcovariate &&
+	C->check != checkMatern && C->check != checkWM
+	) return false;
+  //  printf("B\n");
+  if (equal_coordinate_system(C->Isotropy[C->variants - 1], iso, true)) {
+      if (C->Isotropy[C->variants - 1] > iso &&
+	  C->check != checkpower)  return false;  // see check2x
+      if ((C->Isotropy[C->variants - 1] != iso ||  // see check2x
+	   TypeConsistency(type, C->Typi[C->variants - 1]))) return false;
+    }
+  //  printf("C\n");
+  if (C->Typi[0] > VariogramType && C->Typi[0] != type &&  
+       C->Typi[0] != ShapeType && C->Typi[0] != MathDefinition &&
+      C->check != checktrend) 
+    return false;// see also 
+  //                                     e.g.  newmodel _cov cpy in getNset.cc
+  //  printf("D\n");
+     if (iso== SPHERICAL_ISOTROPIC &&
+	 ((C->finiterange == true && isPosDef(type) && C->vdim == SCALAR)
+	 || C->Monotone == COMPLETELY_MON)) return false;
+     return true;
+}
+
+void AddVariant(Types type, isotropy_type iso) {
+  int nr = currentNrCov - 1;
+  cov_fct *C = CovList + nr; // nicht gatternr
+  assert(addvariantOK(type, iso));
+  C->Typi[C->variants] = type;
+  C->Isotropy[C->variants] = iso;
+  C->variants++;
+}
+
 #define IMPLEMENTED_SEQUENTIAL C->vdim <= 1 &&				\
      (C->domain == PREVMODELD ||	\
-      ((isPosDef(C->Type) || isUndefinedType(C->Type)) && C->domain == XONLY))
+      ((is_any(isPosDef, C) || is_any(UndefinedType, C)) && C->domain == XONLY))
 
 #define IMPLEMENTED_CE							\
-  (((isPosDef(C->Type) || isUndefinedType(C->Type)) && C->domain == XONLY) \
-   || C->domain == PREVMODELD)
+  (((is_any(isPosDef, C) || is_any(UndefinedType, C)) && C->domain == XONLY) \
+  || C->domain == PREVMODELD)
 
 void addCov(int F_derivs, covfct cf, covfct D, covfct D2, covfct inverse,
 	    nonstat_inv nonstat_inverse) {
@@ -1086,7 +1138,7 @@ void addCov(int F_derivs, covfct cf, covfct D, covfct D2, covfct inverse,
 
   assert(nr>=0 && nr<currentNrCov && cf!=NULL);
   cov_fct *C = CovList + nr; // nicht gatternr
-  bool stat_iso = C->domain == XONLY && C->isotropy == ISOTROPIC;
+  bool stat_iso = C->domain == XONLY && C->Isotropy[0] == ISOTROPIC;
 
   C->cov = cf;  
   if (C->RS_derivs < 0) C->RS_derivs = 0;
@@ -1096,8 +1148,8 @@ void addCov(int F_derivs, covfct cf, covfct D, covfct D2, covfct inverse,
     assert(cf != NULL);
     if (C->cov!=NULL && C->RS_derivs < 1) C->RS_derivs = 1;
     C->D=D;    
-    assert(C->isotropy == ISOTROPIC || C->isotropy == SPACEISOTROPIC ||
-	   C->isotropy == PREVMODELI || C->isotropy == PARAM_DEP);
+    assert(C->Isotropy[0] == ISOTROPIC || C->Isotropy[0] == SPACEISOTROPIC ||
+	   C->Isotropy[0] == PREVMODELI || C->Isotropy[0] == PARAM_DEP);
     // C->implemented[TBM2] = NUM_APPROX;
     C->implemented[TBM] = true; 
   }
@@ -1107,7 +1159,7 @@ void addCov(int F_derivs, covfct cf, covfct D, covfct D2, covfct inverse,
    if (C->cov!=NULL && C->D != NULL && C->RS_derivs < 2)  C->RS_derivs = 2;
   }
   if (inverse != NULL) C->inverse = inverse;
-  else if (isMonotone(C->Monotone) && C->isotropy == ISOTROPIC && 
+  else if (isMonotone(C->Monotone) && C->Isotropy[0] == ISOTROPIC && 
 	   C->inverse==ErrCov)
     C->inverse = InverseIsoMon;
   if (stat_iso && C->inverse != ErrInverse) 
@@ -1165,9 +1217,9 @@ void addCov(int F_derivs, covfct cf, covfct D, covfct D2, covfct D3, covfct D4,
   } else {
     C->RS_derivs = 4;
     C->D4 = D4;
-    C->F_derivs = F_derivs >= 0 ?  F_derivs : C->RS_derivs;
-    assert(C->F_derivs <= C->RS_derivs);
   }
+  C->F_derivs = F_derivs >= 0 ?  F_derivs : C->RS_derivs;
+  assert(C->F_derivs <= C->RS_derivs);
 }
 
 
@@ -1217,8 +1269,8 @@ void addCov(covfct distrD, covfct logdistrD, nonstat_inv Dinverse,
   int nr = currentNrCov - 1;
   cov_fct *C = CovList + nr; // nicht gatternr
   assert(nr>=0 && nr<currentNrCov);
-  assert(C->domain == STAT_MISMATCH);
-  assert(C->isotropy == ISO_MISMATCH);
+  assert(C->domain == DOMAIN_MISMATCH);
+  assert(C->Isotropy[0] == ISO_MISMATCH);
   assert(distrD != NULL && logdistrD!=NULL && Dinverse != NULL &&
 	 distrP != NULL && distrQ != NULL &&
 	 distrR != NULL);
@@ -1242,7 +1294,7 @@ void addCov(covfct distrD, covfct logdistrD, nonstat_inv Dinverse,
     assert(distrP2sided == NULL && distrR2sided ==NULL);
     C->domain = XONLY;
   }
-  C->isotropy = CARTESIAN_COORD;
+  C->Isotropy[0] = CARTESIAN_COORD;
 }
 
 
@@ -1250,9 +1302,9 @@ void addlogD(covfct logdistrD) {
   int nr = currentNrCov - 1;
   cov_fct *C = CovList + nr; // nicht gatternr
   assert(nr>=0 && nr<currentNrCov);
-  assert(isProcess(C->Type));
+  assert(isProcess(C->Typi[0]));
   assert(C->domain == XONLY);
-  assert(C->isotropy == UNREDUCED);
+  assert(C->Isotropy[0] == UNREDUCED);
   assert(logdistrD!=NULL);
   assert(C->Struct != struct_failed);
   assert( C->Init !=init_failed);
@@ -1315,9 +1367,10 @@ int addFurtherCov(int F_derivs, covfct cf, covfct D, covfct D2) {
     C->D = D;
     C->RS_derivs = 1;
 
-    assert(C->isotropy == ISOTROPIC || C->isotropy == SPACEISOTROPIC ||
-	   C->isotropy == ZEROSPACEISO ||
-	   C->isotropy == PREVMODELI);
+    assert(C->Isotropy[0] == ISOTROPIC || 
+	   C->Isotropy[0] == SPACEISOTROPIC ||
+	   C->Isotropy[0] == ZEROSPACEISO ||
+	   C->Isotropy[0] == PREVMODELI);
  
     //C->implemented[TBM2] = NUM_APPROX;
     C->implemented[TBM] = IMPLEMENTED; 
@@ -1393,6 +1446,7 @@ void nablahess(covfct nabla, covfct hess) {
 
 }
 
+
 void addLocal(getlocalparam coinit, getlocalparam ieinit) {
   int nr = currentNrCov - 1;
   assert(nr>=0 && nr < currentNrCov) ;
@@ -1402,14 +1456,17 @@ void addLocal(getlocalparam coinit, getlocalparam ieinit) {
   if ((C->implemented[CircEmbedIntrinsic] = ieinit != NULL)) {
     assert(C->D2 != NULL);
     C->ieinit = ieinit;
-    if (pref[CircEmbedCutoff] == PREF_NONE) pref[CircEmbedCutoff] = PREF_BEST;
-  } 
-  if ((C->implemented[CircEmbedCutoff] = coinit != NULL)) {
-    C->coinit = coinit;
     if (pref[CircEmbedIntrinsic] == PREF_NONE)
       pref[CircEmbedIntrinsic] = PREF_BEST;
   }
+  if ((C->implemented[CircEmbedCutoff] = coinit != NULL)) {
+    C->coinit = coinit;
+    if (pref[CircEmbedCutoff] == PREF_NONE) pref[CircEmbedCutoff] = PREF_BEST;
+    if (pref[CircEmbedIntrinsic] > 2)
+      pref[CircEmbedIntrinsic] = 2;
+  }
 }
+
 void addCallLocal(altlocalparam alt) {
   int nr = currentNrCov - 1;
   assert(nr>=0 && nr < currentNrCov) ;
@@ -1419,9 +1476,10 @@ void addCallLocal(altlocalparam alt) {
 
 #define ASSERT_TBM  \
   assert(C->domain == PREVMODELD ||					\
-     ((isPosDef(C->Type)|| isUndefinedType(C->Type)) && C->domain == XONLY)); \
+	 ((is_any(isPosDef, C) || is_any(isUndefined, C))		\
+	  && C->domain == XONLY));					\
   assert(C->vdim == SCALAR || C->vdim == SUBMODEL_DEP ||		\
-	 C->vdim == PARAM_DEP)
+	 C->vdim == PARAM_DEP)					
 
 int addTBM(covfct tbm2) {
   // must be called always AFTER addCov !!!!
@@ -1433,8 +1491,8 @@ int addTBM(covfct tbm2) {
   if (tbm2 != NULL) {
     // addTBM is called from the other addTBM's -- so tbm2 might
     // be NULL
-    assert(C->isotropy==ISOTROPIC || C->isotropy==SPACEISOTROPIC || 
-	   C->isotropy==PREVMODELI);
+    assert(C->Isotropy[0]==ISOTROPIC || C->Isotropy[0]==SPACEISOTROPIC || 
+	   C->Isotropy[0]==PREVMODELI);
     assert(C->D != ErrCov);
     C->implemented[TBM] = IMPLEMENTED;
     if (C->pref[TBM] == PREF_NONE) C->pref[TBM] = PREF_BEST;
@@ -1444,18 +1502,18 @@ int addTBM(covfct tbm2) {
   return nr;
 }
 
-void addTBM(covfct tbm2, initfct Init, spectral_do spectral) {
+void addTBM(covfct tbm2, initfct Init, spectral_do spectralDo) {
   int nr = addTBM(tbm2);
   cov_fct *C = CovList + nr; // nicht gatternr
   ASSERT_TBM;
-  C->spectral=spectral;
+  C->spectral=spectralDo;
   C->Init=Init;
   C->implemented[SpectralTBM] = true;
   if (C->pref[SpectralTBM] == PREF_NONE) C->pref[SpectralTBM] = PREF_BEST;
   }
 
-void addTBM(initfct Init, spectral_do spectral) {
-  addTBM((covfct) NULL, Init, spectral);
+void addTBM(initfct Init, spectral_do spectralDo) {
+  addTBM((covfct) NULL, Init, spectralDo);
 }
 	
 
@@ -1463,7 +1521,8 @@ void addSpecific(int cov) {
   int nr = currentNrCov - 1;
   assert((nr>=0) && (nr<currentNrCov));
   cov_fct *X = CovList + cov; // nicht gatternr
-  assert((CovList[nr].Type == ProcessType &&
+  //  printf("%s %d %d %d %d\n", X->name, CovList[nr].Typi[0], ProcessType, CovList[nr].kappas, X->kappas);
+  assert((CovList[nr].Typi[0] == ProcessType &&
 	  CovList[nr].kappas == X->kappas) ||
 	 ( X->check == checkmal && CovList[nr].check==checkmultproc));
   // to do ... und die namen sollten auch gleich sein...
@@ -1499,12 +1558,6 @@ void addHyper(hyper_pp_fct hyper_pp) {
 //  C->implemented[Special] = true;
 //}
 
-void addMarkov(int *variable) {
-  int nr = currentNrCov - 1;
-  cov_fct *C = CovList + nr; // nicht gatternr
-  C->implemented[Markov] = true; 
-  *variable = nr;
-}
 
 void RandomShape(int maxmoments, structfct Struct, initfct Init, 
 		 dofct Do, do_random_fct DoRandom, 
@@ -1585,12 +1638,11 @@ void addGaussMixture(draw_random drawmix,
 }
 
 void addReturns(return_fct Covariance, ext_bool_ret_fct isCovariance, 
-		return_fct CovMatrix, ext_bool_ret_fct isCovMatrix,
-		return_fct InverseCovMatrix,ext_bool_ret_fct isInverseCovMatrix,
+		return_covmat CovMatrix, ext_bool_ret_fct isCovMatrix,
+		tworeturns_fct InverseCovMatrix,
+		ext_bool_ret_fct isInverseCovMatrix,
 		return_fct Variogram, ext_bool_ret_fct isVariogram,
-		return_fct PseudoVariogram, ext_bool_ret_fct isPseudoVariogram,
-		returnX_fct SelectedCovMatrix,
-		ext_bool_ret_fct isSelectedCovMatrix) {
+		return_fct PseudoVariogram, ext_bool_ret_fct isPseudoVariogram) {
   int nr = currentNrCov - 1;
   cov_fct *C = CovList + nr; // nicht gatternr
   if (Covariance!=NULL) {
@@ -1615,11 +1667,6 @@ void addReturns(return_fct Covariance, ext_bool_ret_fct isCovariance,
     C->is_pseudovariogram =
       isPseudoVariogram==NULL ? isTrue : isPseudoVariogram;
   } else assert(isPseudoVariogram==NULL);
-  if (SelectedCovMatrix!=NULL) {
-    C->selectedcovmatrix=SelectedCovMatrix;
-    C->is_selectedcovmatrix = 
-      isSelectedCovMatrix==NULL ? isTrue : isSelectedCovMatrix;
-  } else assert(isSelectedCovMatrix==NULL);
 }
 
 
@@ -1630,7 +1677,7 @@ void Taylor(double c, double pow) {
 
 
   C->TaylorN = 0;
-  if (isPosDef(C->Type) || isUndefinedType(C->Type)) {
+  if (isPosDef(C->Typi[0]) || isUndefined(C->Typi[0])) {
     assert(pow != 0.0);
     C->Taylor[C->TaylorN][TaylorConst] = 1.0;
     C->Taylor[C->TaylorN][TaylorPow] = 0.0;    
@@ -1694,3 +1741,9 @@ void TailTaylor(double t, double tpow, double texpc, double texppow) {
   assert(C->TailN <= MAXTAYLOR);
 }
 
+
+void setptwise(ptwise_type pt) {
+  int
+    nr = currentNrCov - 1;
+  CovList[nr].ptwise_definite = pt;
+}
