@@ -3,6 +3,10 @@
  Authors 
  Martin Schlather, schlather@math.uni-mannheim.de
 
+ Copyright (C) 2011 -- 2015 Marco Oesting & Martin Schlather
+               2015 -- 2017 Martin Schlather
+
+
  Handling of the different possibilities to pass the trend
 
 Note:
@@ -12,9 +16,6 @@ Note:
  * definitions for the random coin method can be found in MPPFcts.cc
  * definitions for genuinely anisotropic or nondomain models are in
    SophisticatedModel.cc; hyper models also in Hypermodel.cc
-
- Copyright (C) 2011 - 2014 Marco Oesting & Martin Schlather
-
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
@@ -44,13 +45,14 @@ z, x z, x^2 z, ...., x^(k-1) z, y z, x y z, x^2 y z, ..., z^k
 #include <R.h>
 #include <Rdefines.h>
 #include <Rinternals.h>
-#include <math.h>
- 
-#include "RF.h"
-#include "primitive.h"
+#include <Rmath.h>
 #include <R_ext/Lapack.h>
 #include <R_ext/Linpack.h>
-#include "Covariance.h"
+
+#include "RF.h"
+#include "primitive.h"
+#include "Operator.h"
+#include "variogramAndCo.h"
 
 
 
@@ -65,36 +67,36 @@ void mixed(double *x, cov_model *cov, double *v) {
   cov_model  *next = cov->sub[0];
   location_type *loc = Loc(cov);
   mixed_storage *s = cov->Smixed;
-  listoftype *X = PLIST(MIXED_X);
   double //distTF, 
     var = 1.0, 
     *covmatrix=NULL;
   int i, err, // Xnrow, 
     //    err=NOERROR,
-    element = P0INT(MIXED_ELMNT),
-    vdim = cov->vdim2[0],
+    vdim = cov->vdim[0],
     vdimsq = vdim * vdim;
-  if (cov->vdim2[0] != cov->vdim2[1]) BUG;
+  if (cov->vdim[0] != cov->vdim[1]) BUG;
 
-  NotProgrammedYet("");
-
- if (cov->nsub == 0) {
+  if (isNegDef(cov) && cov->nsub == 0) {
     for (i=0; i<vdimsq; i++) v[i] = 0.0;
     return;
   }
-  if (X == NULL) { // no X given, but a submodel	
+
+
+  if (!isShape(cov) && !isTrend(cov)) BUG;
+  *v = LP0(MIXED_X) * P0(MIXED_BETA);
+  return; 
+
+  // TO DO
+
+  BUG;
+
+  if (PisNULL(MIXED_X)) { // no X given, but a submodel	
     COV(x, next, v);
+    BUG;
     return;
   }
     
-  if (element < 0 || element >= cov->nrow[MIXED_X]) { 
-    // realex element <0 ??
-    GERR1("Illegal call of the internal function '%s'", NICK(cov)); 
-  }
-
-  // PMI(cov->calling);
-  // crash(cov);
-  NotProgrammedYet("");  // hier irgendwo stehen geblieben
+ 
 
   if (cov->q[MIXED_CONSTANT]) {
     // Xu , u ~ N(0, C)
@@ -104,22 +106,19 @@ void mixed(double *x, cov_model *cov, double *v) {
       var *= PARAM0(sub, DVAR);
       sub=sub->sub[0];
     }
-    listoftype *list= PARAMLIST(next, CONSTANT_M);
-    covmatrix = list->p[element];
-    if (X->ncol[element] != list->nrow[element]) {
+    covmatrix = LPARAM(next, 99999); BUG; // FIX_M);
+    if (LNCOL(MIXED_X) != LPARAMNROW(next, 99999)) { // FIX_M)) {
       GERR("mixed model: X and covariance matrix M do not match");
     }
   } else {    
     // Xu , u ~ random field
     // random effect, geostatistical covariance model
-
     CovarianceMatrix(cov->key, s->mixedcov);
     covmatrix = s->mixedcov;
-  } // sub->nr != CONSTANT
+  } // sub->nr != FIX
 
   // ab hier vdim =1  
-  *v = XkCXtl(X->p[element], covmatrix, 
-	      X->nrow[element], X->ncol[element], 
+  *v = XkCXtl(LP(MIXED_X), covmatrix, LNROW(MIXED_X), LNCOL(MIXED_X),
 	      loc->i_row, loc->i_col);
   *v *= var;
   return;
@@ -130,19 +129,9 @@ void mixed(double *x, cov_model *cov, double *v) {
 }
 
 void mixed_nonstat(double *x, double *y, cov_model *cov, double *v){
-  int 
-    element = P0INT(MIXED_ELMNT);
 
   NotProgrammedYet("");
   
-  if (element < 0 || element >= cov->nrow[MIXED_X]) { 
-    // relax element <0 ??
-    char msg[100];
-    sprintf(msg, "Illegal call of the internal function '%s'", 
-	    NICK(cov)); 
-    error(msg);
-  }
-
 
   if (cov->nsub != 0 && PisNULL(MIXED_X)) { // no X given, but a submodel	
     cov_model *next = cov->sub[0];
@@ -152,29 +141,27 @@ void mixed_nonstat(double *x, double *y, cov_model *cov, double *v){
 
 void covmatrix_mixed(cov_model *cov, double *v) {
   cov_model *sub = cov->sub[0];
-  int element = P0INT(MIXED_ELMNT);
   
-  if (cov->ncol[MIXED_X] == 0 || element < 0) {
+  if (cov->ncol[MIXED_X] == 0) { // || element < 0) {
     CovList[sub->nr].covmatrix(sub, v);
     return;
   }
-  if (element >= cov->nrow[MIXED_X]) {
-    ERR1("value of '%s' is too large", ELEMENT);
-  }
 
-  listoftype *X = PLIST(MIXED_X);
   double *C=NULL;
-  int nrow = X->nrow[element],
-    ncol=X->ncol[element]; 
-  C = (double*) MALLOC(sizeof(double) * ncol * ncol);
+  int nz=0,
+    nrow = LNROW(MIXED_X), 
+    ncol= LNCOL(MIXED_X),
+    ncol2 = ncol * ncol; 
+  C = (double*) MALLOC(sizeof(double) * ncol2);
   if (C==NULL) {
     StandardCovMatrix(cov, v);
     return;
   }
   
   CovList[sub->nr].covmatrix(sub, C);
-  XCXt(X->p[element], C, v, nrow, ncol); BUG;//todo:?reprogramm XCXt with alloc here ?
+  XCXt(LP(MIXED_X), C, v, nrow, ncol); BUG;//todo:?reprogramm XCXt with alloc here ?
   Loc(cov)->totalpoints = nrow;
+  for (int j=0; j<ncol2; j++) nz += C[j] != 0.0;
 
   // PMI(cov);
   
@@ -199,7 +186,7 @@ void covmatrix_mixed(cov_model *cov, double *v) {
   }
   */
   
-  free(C);
+  FREE(C);
 }
 
 int set_mixed_constant(cov_model *cov) {
@@ -215,17 +202,15 @@ int set_mixed_constant(cov_model *cov) {
     *nrow = cov->nrow; 
   listoftype *X = PLIST(MIXED_X);
 
-  cov->q = (double*) MALLOC(sizeof(double) * 1);
-  cov->qlen = 1;
-
-  // PMI(cov);
+  if (cov->q == NULL) QALLOC(1);
 
   while (sub != NULL && isDollar(sub) &&
 	 ((simple = PARAMisNULL(sub, DPROJ)
 	   && (PARAMisNULL(sub, DSCALE) || PARAM0(sub, DSCALE) == 1.0)
 	   && PARAMisNULL(sub, DANISO)))) sub=sub->sub[0];
   
-  if ((cov->q[MIXED_CONSTANT] = sub != NULL && sub->nr == CONSTANT)) {
+  BUG;
+  if ((cov->q[MIXED_CONSTANT] = sub != NULL && 9999)) {// sub->nr == FIX)) {
     //next->delflag = DEL_COV - 6;
     if (isDollar(next) && next->nrow[DVAR]==0) {
       //next->delflag = DEL_COV - 6;
@@ -237,7 +222,9 @@ int set_mixed_constant(cov_model *cov) {
     int constant_size;
     
     for (i=0; i<nrow[MIXED_X]; i++) {
-      constant_size = PARAMLIST(sub, CONSTANT_M)->nrow[i];
+      BUG;
+      constant_size = PARAMLIST(sub, 9999) ///FIX_M)
+	->nrow[i];
       
       if (ncol[MIXED_X] > 0 && X->ncol[i] != constant_size) {
 	SERR7("%ldth matrix '%s' (%d x %d) and (%d x %d) constant matrix '%s' do not match", i, KNAME(MIXED_X), X->nrow[i], X->ncol[i], constant_size, constant_size, NICK(sub));
@@ -271,7 +258,7 @@ char iscovmatrix_mixed(cov_model *cov) {
 }
 
 void kappamixed(int i, cov_model  VARIABLE_IS_NOT_USED *cov, int *nr, int *nc){
-  if (i==MIXED_DIM || i==MIXED_ELMNT) *nc = *nr = 1;
+  if (i==MIXED_DIM) *nc = *nr = 1;
   else if (i==MIXED_BETA || i==MIXED_DIST) {
     *nc = 1;
     *nr = SIZE_NOT_DETERMINED; 
@@ -279,6 +266,8 @@ void kappamixed(int i, cov_model  VARIABLE_IS_NOT_USED *cov, int *nr, int *nc){
   else *nc = *nr = -1;
 }
 int checkmixed(cov_model *cov) {
+
+ 
     // cov_model *sub;
   //location_type *loc=Loc(cov);
   int i, err, 
@@ -287,18 +276,26 @@ int checkmixed(cov_model *cov) {
     *ncol = cov->ncol,
     *nrow = cov->nrow; // taken[MAX DIM],
   char msg[300];
-  listoftype *X = PLIST(MIXED_X);
-  
-  ROLE_ASSERT(ROLE_GAUSS || cov->role == ROLE_COV);
-  for (i=0; i<nkappa; i++) // NEVER; cf kriging.R, for instance
-    if (cov->kappasub[i] != NULL) SERR("parameters may not be random")
 
-  cov->vdim2[0] = cov->vdim2[1] = 1; //falls kein submodel vorhanden (Marco)
+  //printf("%d\n", MIXED_X);
+  listoftype *X = PLIST(MIXED_X);
+
+  if (!isCartesian(cov->isoown)) return ERRORNOTCARTESIAN;
+
+  if (cov->nrow[0] * cov->ncol[0] != 1 || cov->nrow[1] * cov->ncol[1] != 1)
+    SERR("currently only constant mean possible");
+
+  ROLE_ASSERT(ROLE_GAUSS || cov->role == ROLE_COV);
+  for (i=0; i<nkappa; i++) {// NEVER; cf kriging.R, for instance
+    if (cov->kappasub[i] != NULL) SERR("parameters may not be random");
+    if (cov->ncol[i] * cov->nrow[i] > (i < 2))
+      SERR("current version does not allow for mxied effects as this part of the package is being rewritten."); // SELECT ebenso loeschen?!
+  }
+
+  cov->vdim[0] = cov->vdim[1] = 1; //falls kein submodel vorhanden (Marco)
   cov->maxdim=INFDIM;
   cov->matrix_indep_of_x = true;
 
-  kdefault(cov, MIXED_ELMNT, 0);
-
   if (ncol[MIXED_BETA] > 0) { // b is given  
     if (nsub != 0) 
       SERR("b and a covariance model may not be given at the same time");
@@ -306,7 +303,7 @@ int checkmixed(cov_model *cov) {
 				  KNAME(MIXED_BETA), KNAME(MIXED_X));
     for (i=0; i<nrow[MIXED_X]; i++) {
       if (X->ncol[i] != nrow[MIXED_BETA]) {
-	sprintf(msg,
+	SPRINTF(msg,
 		"%dth set: (%d x %d) matrix '%s' and (%d x %d) vector '%s' do not match",
 		i, X->nrow[0], X->ncol[i], KNAME(MIXED_X),
 		nrow[MIXED_BETA], ncol[MIXED_BETA], KNAME(MIXED_BETA));
@@ -339,13 +336,13 @@ int checkmixed(cov_model *cov) {
       return err;
     }
     
-    // warning("some checks in model `mixed' are missing");
+    // warning("some checks in model 'mixed' are missing");
     // ob X mit C zusammengeht.
     
     setbackward(cov, next);
   }
 
-  if (cov->vdim2[0] > 1) 
+  if (cov->vdim[0] > 1) 
     SERR("multivariate version of mixed not programmed yet");
 
   if (PisNULL(MIXED_DIST) xor PisNULL(MIXED_DIM))
@@ -362,17 +359,9 @@ int checkmixed(cov_model *cov) {
 }
 
 
-  void rangemixed(cov_model  VARIABLE_IS_NOT_USED *cov, range_type *range){
+void rangemixed(cov_model  VARIABLE_IS_NOT_USED *cov, range_type *range){
   int i;
 
-  range->min[MIXED_ELMNT] = 0;
-  range->max[MIXED_ELMNT] = MAXELEMENTS;
-  range->pmin[MIXED_ELMNT] = 0;
-  range->pmax[MIXED_ELMNT] = MAXELEMENTS;
-  range->openmin[MIXED_ELMNT] = false;
-  range->openmax[MIXED_ELMNT] = false;
-
-
   for (i=MIXED_X; i<=MIXED_COORD; i++) {
     range->min[i] = RF_NEGINF;
     range->max[i] = RF_INF;
@@ -394,9 +383,10 @@ int checkmixed(cov_model *cov) {
 
 
 int initmixed(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *S) {
+  int store = GLOBAL.general.set;
   location_type *loc = Loc(cov);
   mixed_storage *s;
-  char errorloc_save[nErrorLoc];
+  errorloc_type errorloc_save;
   double 
     *b = P(MIXED_BETA),
     *coord = P(MIXED_COORD),
@@ -404,18 +394,17 @@ int initmixed(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *S) {
   int 
     Cn = -1, 
     Cdim = -1, 
-    list_element=0,
-    err = NOERROR,
-    vdim = cov->vdim2[0],
+     err = NOERROR,
+    vdim = cov->vdim[0],
     dim = coord!=NULL ? cov->ncol[MIXED_COORD] : P0INT(MIXED_DIM);
   long
     totalpoints = loc->totalpoints,
     total = vdim * totalpoints;
 
-  listoftype *X = PLIST(MIXED_X);
+  //  listoftype *X = PLIST(MIXED_X);
   bool distTF;
 
-  assert(cov->vdim2[0] == cov->vdim2[1]);
+  assert(cov->vdim[0] == cov->vdim[1]);
 
   return ERRORFAILED; // muss in zerlegt werden in init und struct
   // und unten struct aufrufen richtig praeparieren.
@@ -425,36 +414,35 @@ int initmixed(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *S) {
   
   // cholesky zerlegung + bereitstellung von b
 
-  strcpy(errorloc_save, ERROR_LOC);
-  sprintf(ERROR_LOC, "%s%s: ", errorloc_save, "init mixed model");
+  STRCPY(errorloc_save, ERROR_LOC);
+  SPRINTF(ERROR_LOC, "%s%s: ", errorloc_save, "init mixed model");
   
-  assert(cov->nr = MIXEDEFFECT);
+  assert(cov->nr == MIXEDEFFECT);
 
-  NEW_STORAGE(Smixed, MIXED, mixed_storage);
+  NEW_STORAGE(mixed);
   s = cov->Smixed;
 
   NotProgrammedYet("");
 
-  if (s->Xb != NULL) free(s->Xb);
+  FREE(s->Xb);
   if (cov->ncol[MIXED_BETA] > 0) { // b is given
     // X is given, but no covariance model
     if (cov->nrow[MIXED_X] > 1) {
       warning("using first element of list X only");
     }
     
-    if ((s->Xb = (double*) MALLOC(sizeof(double) * X->nrow[list_element])) == 
+    if ((s->Xb = (double*) MALLOC(sizeof(double) * LNROW(MIXED_X))) == 
 	NULL) { err=ERRORMEMORYALLOCATION; goto ErrorHandling; }  
    
-    Ax(X->p[list_element], b, X->nrow[list_element], X->ncol[list_element], 
-       s->Xb);
+    Ax(LP(MIXED_X), b, LNROW(MIXED_X), LNCOL(MIXED_X), s->Xb);
 
   } else { // submodel is given
 
-    NotProgrammedYet(""); //if (cov->q[MIXED_CONSTANT]) error("not 'constant'").
+    NotProgrammedYet(""); //if (cov->q[MIXED_CONSTANT]) ERR("not 'constant'").
 
     cov_model *sub, *next = cov->sub[0];
     
-    if ((s->Xb = (res_type*) MALLOC(sizeof(res_type) * total)) == 0) {
+    if ((s->Xb = (double*) MALLOC(sizeof(double) * total)) == 0) {
       err=ERRORMEMORYALLOCATION; goto ErrorHandling;
     }  
  
@@ -462,10 +450,10 @@ int initmixed(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *S) {
       // Xu , u ~ geostatmodel, u i.a. viel laenger als X (tierzucht)
       // stehende Vektoren, aneinandergereiht
       Cn = cov->ncol[MIXED_DIST];
-      Cn = (int) (0.5 + 0.5 * (1.0 + sqrt(1 + 8.0 * Cn)));
+      Cn = (int) (0.5 + 0.5 * (1.0 + SQRT(1 + 8.0 * Cn)));
       Cdim = cov->nrow[MIXED_DIST];
       assert(Cdim==1);
-      if ((err = covcpy(&(cov->key), next, dist, NULL, dim, dim, Cn, 
+      if ((err = covCpy(&(cov->key), next, dist, NULL, dim, dim, Cn, 
 			false /*Time */,
 			false, distTF)) != NOERROR) goto ErrorHandling;
     } else if (coord != NULL) {
@@ -476,23 +464,25 @@ int initmixed(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *S) {
       
       if (Cdim > Cn)
 	warning("The dimension of the coordinates is higher than the number of points");
-      if ((err = covcpy(&(cov->key), next)) != NOERROR) goto ErrorHandling;
+      if ((err = covCpy(&(cov->key), next)) != NOERROR) goto ErrorHandling;
     } else {
       // Xu , u ~ geostatmodel, X quadratisch	
       Cn = loc->totalpoints;
-      if ((err = covcpy(&(cov->key), next, coord, NULL, dim, dim, Cn, false, 
+
+      BUG; // C HECK whether cov_list = false is ok
+      if ((err = covCpy(&(cov->key), next, coord, NULL, dim, dim, Cn, false, 
 			false, distTF)) != NOERROR) goto ErrorHandling;
     }
     if (cov->key->nr != GAUSSPROC) addModel(&(cov->key), GAUSSPROC);   
     cov->key->calling = cov->key;
     
-    NEW_STORAGE(stor, STORAGE, gen_storage);
-    if ((err = INIT(cov->key, 0, cov->stor)) != NOERROR) goto ErrorHandling;
+    NEW_STORAGE(gen);
+    if ((err = INIT(cov->key, 0, cov->Sgen)) != NOERROR) goto ErrorHandling;
    
     int Xnrow, Xncol;
-    Xnrow = Xncol = Cn * sub->vdim2[0];
+    Xnrow = Xncol = Cn * sub->vdim[0];
     if (loc->i_row==0 && loc->i_col==0) {
-      if (s->mixedcov != NULL) free(s->mixedcov);   
+      FREE(s->mixedcov);   
       s->mixedcov = (double*) MALLOC(sizeof(double) * Xnrow * Xnrow);
       if (s->mixedcov == NULL) {
 	err = ERRORMEMORYALLOCATION;
@@ -505,49 +495,42 @@ int initmixed(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *S) {
   FieldReturn(cov);
 
  ErrorHandling: 
+  GLOBAL.general.set = store;
   cov->initialised = err == NOERROR;
   return err;
 }
 
 
-static int keeprandomeffect = false;
+static int keepRandomEffect = false;
 void domixed(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *S){
   location_type *loc = Loc(cov);
   mixed_storage *s = cov->Smixed;
   double *res  = cov->rf;
   int 
-    list_element=0,
-    vdim = cov->vdim2[0];
+    vdim = cov->vdim[0];
   long i,
     totalpoints = loc->totalpoints,
     total = vdim * totalpoints;
-   assert(cov->vdim2[0] == cov->vdim2[1]);
-
-  listoftype 
-    *X = PLIST(MIXED_X);
+   assert(cov->vdim[0] == cov->vdim[1]);
 
   if (cov->ncol[MIXED_BETA] > 0) { // b is given
     // X is given, but no covariance model
-    if (total == X->nrow[list_element]) {
+    if (total == LNROW(MIXED_X)) {
       for (i=0; i<total; i++) res[i] = s->Xb[i];
     } else {
-      assert(X->nrow[list_element] == 1);
+      assert(LNROW(MIXED_X) == 1);
       for (i=0; i<total; i++) res[i] = s->Xb[0];
-    }
-     //     print("%f\n", res[0]); assert(false);
-    
+    }    
   } else { // submodel is given
     cov_model *key = cov->key;
-    if (!keeprandomeffect || !s->initialized) {
-      do_gaussprocess(key, cov->stor);
-    }
-     
-    if (X != NULL) {
-      AxResType(X->p[list_element], cov->key->rf, X->nrow[list_element], 
-		X->ncol[list_element], res); 
-    } else {
-      res_type *rf = cov->key->rf;
+    if (!keepRandomEffect || !s->initialized) {
+      do_gaussprocess(key, cov->Sgen);
+    }     
+    if (PisNULL(MIXED_X)) {
+      double *rf = cov->key->rf;
       for (i=0; i<total; i++) res[i] = rf[i];
+    } else {
+      AxResType(LP(MIXED_X), cov->key->rf, LNROW(MIXED_X), LNCOL(MIXED_X), res);
     } 
   }
 }
@@ -556,37 +539,29 @@ void domixed(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *S){
 
 
 
-
-
 //////////////////////////////////////////////////////////////////////
 //    trend
 //////////////////////////////////////////////////////////////////////
 
-void trend(double  VARIABLE_IS_NOT_USED *x, cov_model *cov, double *v){
-  // todo : ueberlegen ob hier nicht der Trend-Wert berechnet werden sollte
-
-  ///BUG; // sollte durch plusStat & plusNonStat abgeblock sein ?
-  // to do: stattdessen sollte hier der Trendwert zurueckgeliefert werden ?
-  // nein: model kann auch nur aus Trend bestehen!
-
+void trend(double *x, cov_model *cov, double *v){
+  cov_model *musub = cov->kappasub[TREND_MEAN];
   int i,
-    vdim = cov->vdim2[0],
+    vdim = cov->vdim[0],
     vSq = vdim * vdim;
-  for (i=0; i<vSq; i++) v[i]=0.0;
-  //print("trend %d %f\n", vSq, *v);
+  double *mu = P(TREND_MEAN);
+
+  if (cov->role == ROLE_COV) {
+    BUG;
+    for (i=0; i<vSq; i++) v[i]=0.0;
+  }
+  if (!isShape(cov->typus) && !isTrend(cov->typus)) BUG;
+
+  if (musub != NULL) {
+    FCTN(x, musub, v);
+  } else for (i=0; i<vdim; i++) v[i]=ISNA(mu[i]) || ISNAN(mu[i]) ? 1.0 :  mu[i];
+  // 1.0 notwendig fuer likelihood berechnung;
 }
 
-void trend_nonstat(double  VARIABLE_IS_NOT_USED *x, double  VARIABLE_IS_NOT_USED *y, cov_model *cov, double *v){
-  int i,
-    vdim = cov->vdim2[0],
-    vSq = vdim * vdim;
- 
-  if (cov->role == ROLE_COV)  for (i=0; i<vSq; i++) v[i]=0.0;
-
-  else ERR("trend is called unexpectately.");
-}
-
-
 void kappatrend(int i, cov_model *cov, int *nr, int *nc){
   // i nummer des parameters
   int k;
@@ -644,26 +619,64 @@ int checktrend(cov_model *cov){
   // if (cov->ncol[TREND_LINEAR] > 0 || cov->ncol[TREND_FCT]>0
   //    || cov->ncol[TREND_PARAM_FCT] > 0)
   //  return(ERRORNOTPROGRAMMED);
+  cov_model 
+    *prev = cov->calling,
+    *musub = cov->kappasub[TREND_MEAN];
   double *mu = P(TREND_MEAN),
     *plane = P(TREND_LINEAR);
-  int i, 
-     nkappa = CovList[cov->nr].kappas,
-   *polydeg = PINT(TREND_POLY),
+  int i,  err,
+    nkappa = CovList[cov->nr].kappas,
+    *polydeg = PINT(TREND_POLY),
     vdim = 0, 
     tsdim= cov->tsdim,
     basislen = 0;
   //SEXP Rx, fctbody, envir;
 
-  ROLE_ASSERT(ROLE_GAUSS || cov->role == ROLE_COV);
-  for (i=0; i<nkappa; i++) // NEVER; cf kriging.R, for instance
-    if (cov->kappasub[i] != NULL) SERR("parameters may not be random")
+  if (! ((isCoordinateSystem(cov->isoown) && musub != NULL)
+	 ||
+	 (isIsotropic(cov->isoown) && musub == NULL)))
+    return ERRORFAILED;
 
-  if (mu != NULL) {
+ 
+  bool ok = (musub != NULL && isCoordinateSystem(cov->isoown) && prev != NULL && 
+	     prev->nr == MULT && isTrend(prev->typus)) 
+    ||   // passt das so alles??
+    (musub == NULL && isIsotropic(cov->isoown) &&
+     (prev == NULL || prev->nr == TRENDEVAL ||
+      (prev->nr == PLUS && (prev->calling == NULL || isProcess(prev->calling) ||
+			    isInterface(prev->calling) || isTrend(prev->typus)))))
+     ;
+  if (!ok) SERR("trend model is misused.");
+
+ 
+  ROLE_ASSERT(ROLE_GAUSS || cov->role == ROLE_COV);
+  for (i=0; i<nkappa; i++) {// NEVER; cf kriging.R, for instance
+    if (cov->kappasub[i] != NULL && (i>0 || isRandom(cov->kappasub[i])))
+      SERR("parameters may not be random or sub");
+  }
+
+  if (musub != NULL) {
+    assert(cov->tsdim == cov->xdimown);
+    if ((err = CHECK(musub, cov->tsdim, cov->xdimown, ShapeType,
+		     XONLY, cov->isoown,
+		     SUBMODEL_DEP, ROLE_BASE)) != NOERROR) return err;
+    if (isRandom(musub->typus)) NotProgrammedYet("mixed effects");
+    vdim = musub->vdim[0];
+    cov->matrix_indep_of_x = false;
+  } else if (mu != NULL) {
+    if (prev != NULL && prev->nr != PLUS && isProcess(prev) && 
+	!isInterface(prev) && prev->nr != TRENDEVAL) {
+      cov->typus = ShapeType;
+     }
     vdim = cov->nrow[TREND_MEAN];
     cov->matrix_indep_of_x = true;
-  }
+  } else SERR("no trend argument given");
+
+  
   
   if (plane != NULL) {
+    SERR("'plane' currently not working. Please contact author if needed");
+
     if(vdim>0 && (vdim != cov->ncol[TREND_LINEAR])) {
       SERR("trend parameters have different multivariate dimensions");
     } else vdim = cov->ncol[TREND_LINEAR];
@@ -671,7 +684,9 @@ int checktrend(cov_model *cov){
   }
   
   if (!PisNULL(TREND_POLY)) {
-    if (vdim>0) {
+    SERR("'poly' currently not working. Please contact author if needed");
+
+     if (vdim>0) {
       if (vdim != cov->nrow[TREND_POLY])
 	SERR("trend parameters have different multivariate dimensions");
       SERR("polynomials and free functions in trend may not be mixed with other trend definitions. Please use a sum of trends.");
@@ -718,15 +733,16 @@ int checktrend(cov_model *cov){
   }
   
   if (vdim <= 0) {
-    vdim = cov->calling->vdim2[0];
+    vdim = prev->vdim[0];
     if (vdim <= 0) 
       SERR("multivariate dimension for trend cannot be determined.");
     PALLOC(TREND_MEAN, vdim, 1);
     for(i=0; i<vdim; i++) P(TREND_MEAN)[i] = 0.0;
   }
-  cov->vdim2[0] = cov->vdim2[1] = vdim;
-  cov->isoown = cov->matrix_indep_of_x ? ISOTROPIC : CARTESIAN_COORD;
-  
+  cov->vdim[0] = cov->vdim[1] = vdim;
+  cov->isoown = cov->matrix_indep_of_x ? IsotropicOf(cov->isoown) 
+    : CoordinateSystemOf(cov->isoown);
+
  return NOERROR;
 
 }
@@ -796,12 +812,18 @@ void GetInternalMeanI(cov_model *cov, int vdim, double *mean){
   int i;
   if (cov->nr == TREND) {
     if (cov->ncol[TREND_MEAN]==1) {
-      if (cov->nrow[TREND_MEAN] != vdim) {
+      if (cov->nrow[TREND_MEAN] != vdim || cov->kappasub[TREND_MEAN] != NULL) {
 	for (i=0; i<vdim; i++) mean[i] = RF_NA;
 	return; // only scalar allowed !
       }
       for (i=0; i<vdim; i++) mean[i] += P(TREND_MEAN)[i];
     } 
+  } else if (cov->nr == CONST && isTrend(cov)) {
+    for (i=0; i<vdim; i++) mean[i] += P(CONST_C)[i];
+  } else if (isTrend(cov)) {
+    if (cov->xdimown < MAXSIMUDIM) {
+      FCTN(ZERO, cov, mean);//to be improved! to do
+    } else for (i=0; i<vdim; i++) mean[i] = RF_NA; 
   }
   if (cov->nr == PLUS || cov->nr == TREND) {
     for (i=0; i<cov->nsub; i++) GetInternalMeanI(cov->sub[i], vdim, mean);
@@ -814,30 +836,46 @@ void GetInternalMean(cov_model *cov, int vdim, double *mean){
   GetInternalMeanI(cov, vdim, mean);
 }
 
+
+
+int binomialcoeff(int n, int k) {
+  //programmed as in wikipedia
+  int i, res=1;
+  
+  if((k < 0) || (k > n)) return 0;
+  if(k > n-k) k = n-k; //symmetry
+  for(i=0; i<k; i++) {
+     res *= n-i;
+     res /= i+1; //two steps because of integer division
+  }
+  return res;
+}
+
+
+/*
+
 int init_trend(cov_model *cov, gen_storage *S) {
   
   long err = NOERROR;
-  trend_storage *s;
   int i, 
     *polydeg = PINT(TREND_POLY),
     basislen=0,
     tsdim = cov->tsdim,
-    vdim = cov->vdim2[0];
+    vdim = cov->vdim[0];
   //SEXP fctformals, argnames;
-  if (cov->vdim2[0] != cov->vdim2[1]) BUG;
+  if (cov->vdim[0] != cov->vdim[1]) BUG;
 
   //assert(false);
 
   ROLE_ASSERT_GAUSS;
    
+  NEW_STORAGE(trend);
+  trend_storage *s = cov->Strend;
+
   if(polydeg != NULL) {
     for(i=0; i<vdim; i++) 
       basislen += binomialcoeff(polydeg[i]+tsdim,tsdim);
   }
-
-  NEW_STORAGE(Strend, TREND, trend_storage);
-  s = cov->Strend;
-
   if ((s->x = (double *) MALLOC(sizeof(double) * tsdim))==NULL ||
       (s->xi = (int *) MALLOC(sizeof(int) * tsdim))==NULL ||
       (s->evalplane = (double *) MALLOC(sizeof(double) * vdim))==NULL || 
@@ -846,11 +884,12 @@ int init_trend(cov_model *cov, gen_storage *S) {
     err=ERRORMEMORYALLOCATION;   
     goto ErrorHandling;
   }
-  
+ 
   if (basislen > 0) poly_basis(cov, S); //generates basis of monomials
   //each row consists of one basis element
   //the j-th column consists of the power of the j-th space-time-dimension
-  
+
+ 
   
   if (!PisNULL(TREND_FCT)) { //hier werden Argumente von arbitraryfct ueberprueft
     NotProgrammedYet("");
@@ -898,26 +937,26 @@ int init_trend(cov_model *cov, gen_storage *S) {
 
 void do_trend(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *s){
   location_type *loc = Loc(cov);
-  char errorloc_save[nErrorLoc];
+  char errorloc_save[nErro rLoc];
   trend_storage *S = cov->Strend;
   //PMI(cov->calling->calling);
   assert(S != NULL);
   double t,
     *mu = P(TREND_MEAN),
-    *plane   = P(TREND_LINEAR),
-    *polycoeff = P(TREND_PARAM_POLY),
+    //  *plane   = P(TREND_LINEAR),
+    //  *polycoeff = P(TREND_PARAM_POLY),
     //    *fctcoeff = P(TREND_PARAM_FCT),
     **xgr = loc->xgr,
-    *x = S->x,
-    *evalplane = S->evalplane;
+    // *x = S->x,
+    // *evalplane = S->evalplane
+    ;
 
   int  v, w, 
     basislen, startindex,
     *polydeg = PINT(TREND_POLY),
-    vdim = cov->vdim2[0],
+    vdim = cov->vdim[0],
     tsdim = cov->tsdim,
     spatialdim = loc->spatialdim,
-    *len = loc->length,
     *xi = S->xi,
     *powmatrix = S->powmatrix;
   //SEXP fctbody, tempres, envir, Rx;
@@ -926,23 +965,25 @@ void do_trend(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *s){
     totalpoints = loc->totalpoints,
     total = totalpoints * vdim;
  
-  res_type *res = cov->rf;
+  double *res = cov->rf;
   
-  assert(cov->vdim2[0] == cov->vdim2[1]);
+  assert(cov->vdim[0] == cov->vdim[1]);
 
-  strcpy(errorloc_save, ERROR_LOC);
-  sprintf(ERROR_LOC, "%s%s: ", errorloc_save, "add trend model");
+  STRCPY(errorloc_save, ERROR_LOC);
+  SPRINTF(ERROR_LOC, "%s%s: ", errorloc_save, "add trend model");
  
   // print("%s\n", ERROR_LOC);
 
-  if (mu != NULL) {  
+  if (cov->kappasub[TREND_MEAN] != NULL) {
+    
+  } else if (mu != NULL) {  
     for (k=0; k<total; ) {
       for (v=0; v<vdim; res[k++] = mu[v++]); 
     } 
   } else {
     for (k=0; k<total; res[k++]=0.0);
   }
-  
+
   if (plane != NULL) {  
     if (loc->grid) {
       for (w=0; w<tsdim; w++)  {
@@ -958,7 +999,7 @@ void do_trend(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *s){
 	i = 0;
 	(xi[i])++;
 	x[i] += xgr[i][XSTEP];
-	while(xi[i]>=len[i]) {
+	while(xi[i]>=loc->xgr[i][XLENGTH]) {
 	  xi[i] = 0;
 	  x[i] = xgr[i][XSTART];
 	  if (i<tsdim-1) {
@@ -972,7 +1013,7 @@ void do_trend(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *s){
       }	
     } else if (loc->Time) {
       long m, 
-	endfor= loc->length[loc->timespacedim-1],
+	endfor= (long int) loc->xgr[loc->timespacedim-1][XLENGTH],
 	endfor2 = loc->spatialtotalpoints;
       for(k=m=0, t=loc->T[XSTART]; m<endfor; m++, t+=loc->T[XSTEP]) {
 	// for(t=loc->T[XSTART], i=0; t < loc->T[XEND]; t += loc->T[XSTEP]) {
@@ -1015,7 +1056,7 @@ void do_trend(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *s){
 	  i = 0;
 	  (xi[i])++;
 	  x[i] += xgr[i][XSTEP];
-	  while(xi[i]>=len[i]) {
+	  while(xi[i]>=xgr[i][XLENGTH]) {
 	    xi[i] = 0;
 	    x[i] = xgr[i][XSTART];
 	    if (i<tsdim-1) {
@@ -1098,7 +1139,7 @@ void do_trend(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *s){
 //          }	
 //      } else if(loc->Time) {
 // 	 int k, 
-// 	   endfor= loc->length[loc->timespacedim-1];
+// 	   endfor= (int) loc->T[XLENGTH];
 // 	 for(k=0, t=loc->T[XSTART], i=0; k<endfor; k++, t += loc->T[XSTEP]) {
 // 	   //	 for(t=loc->T[XSTART], i=0; t < loc->T[XEND]; t+=loc->T[XSTEP]) {
 // 	   for(k=0, j=0; k < loc->spatialtotalpoints; k++, i++) {
@@ -1137,35 +1178,22 @@ void do_trend(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *s){
 //        UNPROTECT(4);
   }  
   
+  
   return;
 
 }
 
-
-int binomialcoeff(int n, int k) {
-  //programmed as in wikipedia
-  int i, res=1;
-  
-  if((k < 0) || (k > n)) return 0;
-  if(k > n-k) k = n-k; //symmetry
-  for(i=0; i<k; i++) {
-     res *= n-i;
-     res /= i+1; //two steps because of integer division
-  }
-  return res;
-}
-
 void poly_basis(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *s) {
   
   trend_storage *S = cov->Strend;
   int basislen=0, powsum, d, i, j, k, v,
     dim = cov->tsdim,
-    vdim = cov->vdim2[0],
+    vdim = cov->vdim[0],
     *powmatrix = S->powmatrix,
       *dimi=NULL,
       err=NOERROR;
   int *polydeg = PINT(TREND_POLY);
-  assert(cov->vdim2[0] == cov->vdim2[1]);
+  assert(cov->vdim[0] == cov->vdim[1]);
   
   dimi = (int *) MALLOC(dim * sizeof(int));
   if (dimi == NULL) {
@@ -1200,8 +1228,7 @@ void poly_basis(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *s) {
   //print("\n");
   
   ErrorHandling:
-  if (dimi != NULL) free(dimi);   
-  dimi = NULL;
+  FREE(dimi);   
   if (err != NOERROR) XERR(err);
    
   return;
@@ -1214,14 +1241,12 @@ double evalpoly(double *x, int *powmatrix, double *polycoeff, int basislen,
   double res = 0, tempres;
   for(j=i=0; i<basislen; i++) {
      tempres=1;
-     for(d=0; d<dim; d++) tempres *= pow(x[d], powmatrix[j++]);
+     for(d=0; d<dim; d++) tempres *= POW(x[d], powmatrix[j++]);
      res += polycoeff[i] * tempres;
   }
   return(res);
 }
 
-
-
 void likelihood_trend(double VARIABLE_IS_NOT_USED *x, double VARIABLE_IS_NOT_USED invmatrix, cov_model *cov, 
 		      double VARIABLE_IS_NOT_USED *v){ 
   if (cov->role == ROLE_GAUSS) {
@@ -1237,3 +1262,60 @@ void likelihood_trend(double VARIABLE_IS_NOT_USED *x, double VARIABLE_IS_NOT_USE
   }
 
 }
+*/
+
+
+
+
+int checkTrendEval(cov_model *cov) { // auch fuer TrendEval
+   cov_model *next = cov->sub[0];
+   int err;
+   //dim =  Gettimespacedim(cov);
+
+  if ((err = CHECK(next, cov->tsdim, cov->xdimown, TrendType,
+		     XONLY, cov->isoown,
+		     SUBMODEL_DEP, ROLE_BASE)) != NOERROR) return err;
+
+  assert(isTrend(cov->sub[0]->typus));
+  setbackward(cov, next);
+  cov->vdim[0] = next->vdim[0]; 
+  cov->vdim[1] = next->vdim[1];
+  if (cov->vdim[0] != 1) NotProgrammedYet("");
+  if ((err = kappaBoxCoxParam(cov, GAUSS_BOXCOX)) != NOERROR) return err;
+
+  return NOERROR;
+}
+ 
+
+int init_TrendEval(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *s){// auch fuer TrendEval
+  int err;
+
+  if (cov->vdim[0] != 1) NotProgrammedYet("");
+
+  if ((err = check_fctn(cov)) != NOERROR) return err;
+  
+  ROLE_ASSERT_GAUSS;
+  err = FieldReturn(cov);
+  cov->simu.active = err == NOERROR;
+  if (PL>= PL_STRUCTURE) PRINTF("\n'%s' is now initialized.\n", NAME(cov));
+  return err;
+}
+
+void do_TrendEval(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *s){
+  double
+    *res = cov->rf;
+  assert(res != NULL);
+  errorloc_type errorloc_save;
+  STRCPY(errorloc_save, ERROR_LOC);
+  SPRINTF(ERROR_LOC, "%s%s: ", errorloc_save, "add trend model");
+  Fctn(NULL, cov, res);
+  STRCPY(ERROR_LOC, errorloc_save);
+
+  BOXCOX_INVERSE;
+  return; 
+}
+
+
+void range_TrendEval(cov_model VARIABLE_IS_NOT_USED *cov, range_type *range){  
+  GAUSS_COMMON_RANGE;
+}
